@@ -18,6 +18,7 @@ from txzookeeper.managed import ManagedClient
 from juju.control.options import setup_twistd_options
 from juju.errors import NoConnection, JujuError
 from juju.lib.zklog import ZookeeperHandler
+from juju.lib.zk import CLIENT_SESSION_TIMEOUT
 from juju.state.environment import GlobalSettingsStateManager
 
 
@@ -204,7 +205,8 @@ class BaseAgent(object, service.Service):
     def connect(self):
         """Return an authenticated connection to the juju zookeeper."""
         yield self._kill_existing_session()
-        self.client = yield ManagedClient().connect(
+        self.client = yield ManagedClient(
+            session_timeout=CLIENT_SESSION_TIMEOUT).connect(
             self.config["zookeeper_servers"])
         save_client_id(
             self.config["session_file"], self.client.client_id)
