@@ -61,7 +61,7 @@ class BaseAgentTest(TestCase):
         self.assertFalse(parser.get_default("nodaemon"))
         self.assertEqual(parser.get_default("rundir"), ".")
         self.assertEqual(parser.get_default("chroot"), None)
-        self.assertEqual(parser.get_default("umask"), None)
+        self.assertEqual(parser.get_default("umask"), '0022')
         self.assertEqual(parser.get_default("uid"), None)
         self.assertEqual(parser.get_default("gid"), None)
         self.assertEqual(parser.get_default("euid"), None)
@@ -322,6 +322,7 @@ class BaseAgentTest(TestCase):
         self.mocker.replay()
 
         agent = BaseAgent()
+        agent.set_watch_enabled(False)
         agent.configure({
             "zookeeper_servers": get_test_zookeeper_address(),
             "juju_directory": self.makeDir(),
@@ -420,6 +421,7 @@ class BaseAgentTest(TestCase):
         self.mocker.replay()
 
         agent = BaseAgent()
+        agent.set_watch_enabled(False)
         parser = argparse.ArgumentParser()
         agent.setup_options(parser)
         options = parser.parse_args(
@@ -454,6 +456,7 @@ class BaseAgentTest(TestCase):
         self.mocker.replay()
 
         agent = BaseAgent()
+        agent.set_watch_enabled(False)
         parser = argparse.ArgumentParser()
         agent.setup_options(parser)
         options = parser.parse_args(
@@ -485,6 +488,7 @@ class BaseAgentTest(TestCase):
         self.mocker.replay()
 
         agent = BaseAgent()
+        agent.set_watch_enabled(False)
         parser = argparse.ArgumentParser()
         agent.setup_options(parser)
         options = parser.parse_args(
@@ -516,6 +520,7 @@ class BaseAgentTest(TestCase):
         self.mocker.replay()
 
         agent = BaseAgent()
+        agent.set_watch_enabled(False)
         parser = argparse.ArgumentParser()
         agent.setup_options(parser)
         options = parser.parse_args(
