@@ -1,18 +1,12 @@
 import logging
 
-import zookeeper
-
 from twisted.internet.defer import inlineCallbacks, fail, succeed
 from twisted.internet import reactor
 
-from txzookeeper.client import ZOO_OPEN_ACL_UNSAFE
-
 from juju.agents.provision import ProvisioningAgent
 from juju.environment.environment import Environment
-from juju.environment.config import EnvironmentsConfig
 from juju.environment.errors import EnvironmentsConfigError
-from juju.environment.tests.test_config import SAMPLE_ENV
-from juju.machine.constraints import Constraints
+from juju.machine.tests.test_constraints import dummy_cs, series_constraints
 from juju.errors import ProviderInteractionError
 from juju.lib.mocker import MATCH
 from juju.providers.dummy import DummyMachine
@@ -37,23 +31,21 @@ class ProvisioningTestBase(AgentTestBase):
         yield super(ProvisioningTestBase, self).setUp()
         self.machine_manager = MachineStateManager(self.client)
 
-    def add_machine_state(self):
+    def add_machine_state(self, constraints=None):
         return self.machine_manager.add_machine_state(
-            Constraints.from_strs("dummy", []).with_series("series"))
-
-    def get_serialized_environment(self):
-        config = EnvironmentsConfig()
-        config.parse(SAMPLE_ENV)
-        return config.serialize("myfirstenv")
+            constraints or series_constraints)
 
 
 class ProvisioningAgentStartupTest(ProvisioningTestBase):
 
+    setup_environment = False
+
     @inlineCallbacks
     def setUp(self):
         yield super(ProvisioningAgentStartupTest, self).setUp()
         yield self.agent.connect()
 
+    @inlineCallbacks
     def test_agent_waits_for_environment(self):
         """
         When the agent starts it waits for the /environment node to exist.
@@ -61,31 +53,20 @@ class ProvisioningAgentStartupTest(ProvisioningTestBase):
         deserialize it into an environment object.
         """
         env_loaded_deferred = self.agent.configure_environment()
-
-        def verify_environment(result):
-            self.assertTrue(isinstance(result, Environment))
-            self.assertEqual(result.name, "myfirstenv")
-
-        env_loaded_deferred.addCallback(verify_environment)
-
-        def create_environment_node():
-            self.assertFalse(env_loaded_deferred.called)
-            return self.client.create(
-                "/environment", self.get_serialized_environment())
-
-        reactor.callLater(0.3, create_environment_node)
-        return env_loaded_deferred
+        reactor.callLater(
+            0.3, self.push_default_config, with_constraints=False)
+        result = yield env_loaded_deferred
+        self.assertTrue(isinstance(result, Environment))
+        self.assertEqual(result.name, "firstenv")
 
     @inlineCallbacks
     def test_agent_with_existing_environment(self):
         """An agent should load an existing environment to configure itself."""
-
-        yield self.client.create(
-            "/environment", self.get_serialized_environment())
+        yield self.push_default_config()
 
         def verify_environment(result):
             self.assertTrue(isinstance(result, Environment))
-            self.assertEqual(result.name, "myfirstenv")
+            self.assertEqual(result.name, "firstenv")
 
         d = self.agent.configure_environment()
         d.addCallback(verify_environment)
@@ -103,15 +84,13 @@ class ProvisioningAgentStartupTest(ProvisioningTestBase):
         while the agent is processing the NoNodeException, it should detect
         this and configure normally.
         """
-        data = self.get_serialized_environment()
         exists_and_watch = self.agent.client.exists_and_watch
 
         mock_client = self.mocker.patch(self.agent.client)
         mock_client.exists_and_watch("/environment")
 
         def inject_creation(path):
-            zookeeper.create(
-                self.agent.client.handle, path, data, [ZOO_OPEN_ACL_UNSAFE])
+            self.push_default_config(with_constraints=False)
             return exists_and_watch(path)
 
         self.mocker.call(inject_creation)
@@ -131,8 +110,6 @@ class ProvisioningAgentTest(ProvisioningTestBase):
     @inlineCallbacks
     def setUp(self):
         yield super(ProvisioningAgentTest, self).setUp()
-        yield self.client.create(
-            "/environment", self.get_serialized_environment())
         self.agent.set_watch_enabled(False)
         yield self.agent.startService()
         self.output = self.capture_logging("juju.agents.provision",
@@ -268,8 +245,7 @@ class ProvisioningAgentTest(ProvisioningTestBase):
         If the environment changes the agent reconfigures itself
         """
         provider = self.agent.provider
-        data = self.get_serialized_environment()
-        yield self.client.set("/environment", data)
+        yield self.push_default_config()
         yield self.sleep(0.2)
         self.assertNotIdentical(provider, self.agent.provider)
 
@@ -314,14 +290,22 @@ class ProvisioningAgentTest(ProvisioningTestBase):
         If there's an error when processing changes, the agent should log
         the error and continue.
         """
-        machine_state0 = yield self.add_machine_state()
-        machine_state1 = yield self.add_machine_state()
+        machine_state0 = yield self.add_machine_state(
+            dummy_cs.parse(["cpu=10"]).with_series("series"))
+        machine_state1 = yield self.add_machine_state(
+            dummy_cs.parse(["cpu=20"]).with_series("series"))
 
         mock_provider = self.mocker.patch(self.agent.provider)
-        mock_provider.start_machine({"machine-id": 0})
+        mock_provider.start_machine({
+            "machine-id": 0, "constraints": {
+                "arch": "amd64", "cpu": 10, "mem": 512,
+                "provider-type": "dummy", "ubuntu-series": "series"}})
         self.mocker.result(fail(ProviderInteractionError()))
 
-        mock_provider.start_machine({"machine-id": 1})
+        mock_provider.start_machine({
+            "machine-id": 1, "constraints": {
+                "arch": "amd64", "cpu": 20, "mem": 512,
+                "provider-type": "dummy", "ubuntu-series": "series"}})
         self.mocker.passthrough()
         self.mocker.replay()
 
@@ -476,8 +460,6 @@ class FirewallManagerTest(
     @inlineCallbacks
     def setUp(self):
         yield super(FirewallManagerTest, self).setUp()
-        yield self.client.create(
-            "/environment", self.get_serialized_environment())
         self.agent.set_watch_enabled(False)
         yield self.agent.startService()
 
@@ -509,11 +491,14 @@ class FirewallManagerTest(
         # Modify services, while subsequently poking to ensure service
         # watch is processed on each modification
         yield self.add_service("wordpress")
-        yield self.poke_zk()
+        while len(seen) < 1:
+            yield self.poke_zk()
         mysql = yield self.add_service("mysql")
-        yield self.poke_zk()
+        while len(seen) < 2:
+            yield self.poke_zk()
         yield self.service_state_manager.remove_service_state(mysql)
-        yield self.poke_zk()
+        while len(seen) < 3:
+            yield self.poke_zk()
 
         self.assertEqual(
             seen,
