@@ -9,7 +9,9 @@ from juju.agents.base import TwistedOptionNamespace
 from juju.charm import get_charm_from_path
 from juju.charm.url import CharmURL
 from juju.errors import JujuError
+from juju.hooks.executor import HookExecutor
 from juju.lib import serializer
+
 from juju.state.environment import GlobalSettingsStateManager
 from juju.state.errors import ServiceStateNotFound
 from juju.state.service import NO_HOOKS, RETRY_HOOKS
@@ -30,11 +32,15 @@ class UnitAgentTestBase(AgentTestBase, WorkflowTestBase):
 
     @inlineCallbacks
     def setUp(self):
+        self.patch(HookExecutor,
+                   "LOCK_PATH",
+                   os.path.join(self.makeDir(), "hook.lock"))
         yield super(UnitAgentTestBase, self).setUp()
         settings = GlobalSettingsStateManager(self.client)
         yield settings.set_provider_type("dummy")
         self.change_environment(
             PATH=get_cli_environ_path(),
+            JUJU_ENV_UUID="snowflake",
             JUJU_UNIT_NAME="mysql/0")
 
     @inlineCallbacks
