@@ -118,6 +118,7 @@ class RemoteCharmRepository(object):
         self.url_base = url_base
         if cache_path is not None:
             self.cache_path = cache_path
+        self.no_stats = bool(os.environ.get("JUJU_TESTING"))
 
     def __str__(self):
         return "charm store"
@@ -127,9 +128,13 @@ class RemoteCharmRepository(object):
         charm_id = str(charm_url)
         url = "%s/charm-info?charms=%s" % (
             self.url_base, urllib.quote(charm_id))
+        if self.no_stats:
+            url += "&stats=0"
         try:
             host = urlparse.urlparse(url).hostname
-            all_info = json.loads((yield getPage(url, contextFactory=VerifyingContextFactory(host))))
+            all_info = json.loads(
+                (yield getPage(
+                    url, contextFactory=VerifyingContextFactory(host))))
             charm_info = all_info[charm_id]
             for warning in charm_info.get("warnings", []):
                 log.warning("%s: %s", charm_id, warning)
@@ -151,8 +156,13 @@ class RemoteCharmRepository(object):
         f.close()
         downloading_path = f.name
         host = urlparse.urlparse(url).hostname
+        if self.no_stats:
+            url += "?stats=0"
         try:
-            yield downloadPage(url, downloading_path, contextFactory=VerifyingContextFactory(host))
+            yield downloadPage(
+                url,
+                downloading_path,
+                contextFactory=VerifyingContextFactory(host))
         except Error:
             raise CharmNotFound(self.url_base, charm_url)
         os.rename(downloading_path, cache_path)
