@@ -354,6 +354,13 @@ class RemoteRepositoryTest(RepositoryTestBase):
         d.addCallback(verify)
         return d
 
+    def test_find_plain_uncached_no_stat(self):
+        self.change_environment(JUJU_TESTING="1")
+        return self.assert_find_uncached(
+            "https://somewhe.re", "cs:series/name",
+            "https://somewhe.re/charm-info?charms=cs%3Aseries/name&stats=0",
+            "https://somewhe.re/charm/series/name-1?stats=0")
+
     def test_find_plain_uncached(self):
         return self.assert_find_uncached(
             "https://somewhe.re", "cs:series/name",
