@@ -1,11 +1,11 @@
 import argparse
-
 import yaml
 
 from twisted.internet.defer import inlineCallbacks
 
 from juju.charm.errors import ServiceConfigValueError
 from juju.control.utils import get_environment
+from juju.lib import serializer
 from juju.lib.format import get_charm_formatter
 from juju.state.service import ServiceStateManager
 
@@ -62,7 +62,7 @@ def command(options):
 
         yaml_data = options.config.read()
         try:
-            data = yaml.safe_load(yaml_data)
+            data = serializer.yaml_load(yaml_data)
         except yaml.YAMLError:
             raise ServiceConfigValueError(
                 "Config file %r not valid YAML" % options.config.name)
@@ -105,8 +105,7 @@ def config_set(environment, service_name, service_options):
         options = formatter.parse_keyvalue_pairs(service_options)
 
     config = yield charm.get_config()
-    # ignore the output of validate, we run it so it might throw an exception
-    config.validate(options)
+    options = config.validate(options)
 
     # Apply the change
     state = yield service.get_config()
