@@ -1,10 +1,8 @@
-import yaml
-
 from twisted.internet.defer import inlineCallbacks
 
 from juju.control import main
-
 from juju.charm.tests import local_charm_id
+from juju.lib import serializer
 from .common import MachineControlToolTest
 
 
@@ -13,7 +11,7 @@ class ControlJujuGetTest(MachineControlToolTest):
     @inlineCallbacks
     def setUp(self):
         yield super(ControlJujuGetTest, self).setUp()
-        self.stderr = self.capture_stream("stderr")
+        self.output = self.capture_logging()
 
     @inlineCallbacks
     def test_get_service_config(self):
@@ -32,7 +30,7 @@ class ControlJujuGetTest(MachineControlToolTest):
         main(["get", "wordpress"])
 
         yield finished
-        data = yaml.load(output.getvalue())
+        data = serializer.yaml_load(output.getvalue())
         self.assertEqual(
             {"service": "wordpress",
              "charm": "local:series/wordpress-3",
@@ -59,7 +57,7 @@ class ControlJujuGetTest(MachineControlToolTest):
         main(["get", "dummy"])
 
         yield finished
-        data = yaml.load(output.getvalue())
+        data = serializer.yaml_load(output.getvalue())
 
         self.assertEqual(
             {"service": "dummy",
@@ -68,7 +66,7 @@ class ControlJujuGetTest(MachineControlToolTest):
                  'outlook': {
                      'description': 'No default outlook.',
                      'type': 'string',
-                     'value': '-Not set-'},
+                     'value': None},
                  'skill-level': {
                      'description': 'A number indicating skill.',
                      'value': 24,
@@ -97,4 +95,4 @@ class ControlJujuGetTest(MachineControlToolTest):
         yield finished
 
         self.assertIn("Service 'whatever' was not found",
-                      self.stderr.getvalue())
+                      self.output.getvalue())
