@@ -60,7 +60,7 @@ class ControlDebugHookTest(
         self.assertIn(
             "Charm 'local:series/mysql-1' does not contain hook "
             "'bad-happened'",
-            self.stderr.getvalue())
+            self.output.getvalue())
 
     @inlineCallbacks
     def test_debug_hook_invalid_unit_name(self):
@@ -73,7 +73,7 @@ class ControlDebugHookTest(
         yield finished
         self.assertIn(
             "Service unit 'mysql/42' was not found",
-            self.stderr.getvalue())
+            self.output.getvalue())
 
     @inlineCallbacks
     def test_debug_hook_invalid_service(self):
@@ -86,7 +86,7 @@ class ControlDebugHookTest(
         yield finished
         self.assertIn(
             "Service 'magic' was not found",
-            self.stderr.getvalue())
+            self.output.getvalue())
 
     @inlineCallbacks
     def test_debug_hook_unit_agent_not_available(self):
