@@ -1,7 +1,5 @@
 import logging
 import os
-import yaml
-
 
 from twisted.internet.defer import inlineCallbacks, succeed
 
@@ -15,6 +13,7 @@ from juju.charm.directory import CharmDirectory
 from juju.charm.repository import RemoteCharmRepository
 from juju.charm.url import CharmURL
 from juju.charm.errors import ServiceConfigValueError
+from juju.lib import serializer
 
 from juju.state.charm import CharmStateManager
 from juju.state.environment import EnvironmentStateManager
@@ -32,7 +31,6 @@ class ControlDeployTest(MachineControlToolTest):
     @inlineCallbacks
     def setUp(self):
         yield super(ControlDeployTest, self).setUp()
-
         config = {
             "environments": {
                 "firstenv": {
@@ -40,8 +38,7 @@ class ControlDeployTest(MachineControlToolTest):
                     "admin-secret": "homer",
                     "placement": "unassigned",
                     "default-series": "series"}}}
-        self.write_config(yaml.dump(config))
-        self.config.load()
+        yield self.push_config("firstenv", config)
 
     def test_deploy_multiple_environments_none_specified(self):
         """
@@ -60,8 +57,8 @@ class ControlDeployTest(MachineControlToolTest):
                 "secondenv": {
                     "type": "dummy", "admin-secret": "marge"}}}
 
-        self.write_config(yaml.dump(config))
-        stderr = self.capture_stream("stderr")
+        self.write_config(serializer.dump(config))
+        stderr = self.capture_logging()
         main(["deploy", "--repository", self.unbundled_repo_path, "mysql"])
         self.assertIn("There are multiple environments", stderr.getvalue())
 
@@ -71,7 +68,7 @@ class ControlDeployTest(MachineControlToolTest):
         self.setup_exit(1)
         self.mocker.replay()
 
-        stderr = self.capture_stream("stderr")
+        stderr = self.capture_logging()
         main(["deploy", "local:redis"])
         self.assertIn("No repository specified", stderr.getvalue())
 
@@ -84,7 +81,7 @@ class ControlDeployTest(MachineControlToolTest):
         self.setup_exit(1)
         self.mocker.replay()
 
-        stderr = self.capture_stream("stderr")
+        stderr = self.capture_logging()
         main([
             "deploy", "local:redis"])
         self.assertNotIn("No repository specified", stderr.getvalue())
@@ -95,7 +92,7 @@ class ControlDeployTest(MachineControlToolTest):
         self.setup_exit(1)
         self.mocker.replay()
 
-        stderr = self.capture_stream("stderr")
+        stderr = self.capture_logging()
         main([
             "deploy", "--repository", self.unbundled_repo_path, "local:redis"])
         self.assertIn(
@@ -108,7 +105,7 @@ class ControlDeployTest(MachineControlToolTest):
         self.setup_exit(1)
         self.mocker.replay()
 
-        stderr = self.capture_stream("stderr")
+        stderr = self.capture_logging()
         main([
             "deploy", "--repository", self.unbundled_repo_path, "local:sample",
             "--constraints", "arch=arm tweedledee tweedledum"])
@@ -169,9 +166,9 @@ class ControlDeployTest(MachineControlToolTest):
                 "secondenv": {
                     "type": "dummy", "admin-secret": "marge"}}}
 
-        self.write_config(yaml.dump(config))
+        self.write_config(serializer.dump(config))
 
-        stderr = self.capture_stream("stderr")
+        stderr = self.capture_logging()
         main(["deploy", "--environment", "roman-candle",
               "--repository", self.unbundled_repo_path, "sample"])
         self.assertIn("Invalid environment 'roman-candle'", stderr.getvalue())
@@ -188,7 +185,7 @@ class ControlDeployTest(MachineControlToolTest):
                 "secondenv": {
                     "type": "dummy", "admin-secret": "marge"}}}
 
-        self.write_config(yaml.dump(config))
+        self.write_config(serializer.dump(config))
 
         def match_config(config):
             return isinstance(config, EnvironmentsConfig)
@@ -228,7 +225,7 @@ class ControlDeployTest(MachineControlToolTest):
         self.assertEquals(charm_id, "local:series/sample-2")
         constraints = yield service_state.get_constraints()
         expect_constraints = {
-            "arch": None, "cpu": 123, "mem": 512,
+            "arch": "amd64", "cpu": 123, "mem": 512,
             "provider-type": "dummy", "ubuntu-series": "series"}
         self.assertEquals(constraints, expect_constraints)
 
@@ -355,8 +352,8 @@ class ControlDeployTest(MachineControlToolTest):
         env_state_manager = EnvironmentStateManager(self.client)
         env_config = yield env_state_manager.get_config()
 
-        self.assertEquals(yaml.load(env_config.serialize("firstenv")),
-                          yaml.load(self.config.serialize("firstenv")))
+        self.assertEquals(serializer.load(env_config.serialize("firstenv")),
+                          serializer.load(self.config.serialize("firstenv")))
 
     @inlineCallbacks
     def test_deploy_reuses_machines(self):
@@ -381,7 +378,7 @@ class ControlDeployTest(MachineControlToolTest):
 
     def test_deploy_missing_config(self):
         """Missing config files should prevent the deployment"""
-        stderr = self.capture_stream("stderr")
+        stderr = self.capture_logging()
         self.setup_cli_reactor()
         self.setup_exit(1)
         self.mocker.replay()
@@ -396,7 +393,7 @@ class ControlDeployTest(MachineControlToolTest):
         """Valid config options should be available to the deployed
         service."""
         config_file = self.makeFile(
-            yaml.dump(dict(otherservice=dict(application_file="foo"))))
+            serializer.dump(dict(otherservice=dict(application_file="foo"))))
         environment = self.config.get("firstenv")
 
         failure = deploy.deploy(
@@ -410,7 +407,7 @@ class ControlDeployTest(MachineControlToolTest):
     def test_deploy_with_invalid_config(self):
         """Can't deploy with config that doesn't pass charm validation."""
         config_file = self.makeFile(
-            yaml.dump(dict(myblog=dict(application_file="foo"))))
+            serializer.dump(dict(myblog=dict(application_file="foo"))))
         environment = self.config.get("firstenv")
 
         failure = deploy.deploy(
@@ -428,7 +425,7 @@ class ControlDeployTest(MachineControlToolTest):
     def test_deploy_with_config(self):
         """Valid config options should be available to the deployed
         service."""
-        config_file = self.makeFile(yaml.dump(dict(
+        config_file = self.makeFile(serializer.dump(dict(
                     myblog=dict(outlook="sunny",
                                 username="tester01"))))
         environment = self.config.get("firstenv")
@@ -490,9 +487,7 @@ class ControlDeployTest(MachineControlToolTest):
                     "placement": "local",
                     "type": "dummy",
                     "default-series": "series"}}}
-
-        self.write_config(yaml.dump(config))
-        self.config.load()
+        yield self.push_config("firstenv", config)
 
         finished = self.setup_cli_reactor()
         self.setup_exit(0)
@@ -508,3 +503,93 @@ class ControlDeployTest(MachineControlToolTest):
         unit = units[0]
         machine_id = yield unit.get_assigned_machine_id()
         self.assertEqual(machine_id, 0)
+
+    @inlineCallbacks
+
+    def test_deploy_informs_with_subordinate(self):
+        """Verify subordinate charm doesn't deploy.
+
+        And that it properly notifies the user.
+        """
+        log = self.capture_logging()
+        finished = self.setup_cli_reactor()
+        self.setup_exit(0)
+        self.mocker.replay()
+
+        # missing config file
+        main(["deploy",
+            "--repository", self.unbundled_repo_path, "local:logging"])
+        yield finished
+        self.assertIn(
+            "Subordinate 'logging' awaiting relationship to "
+            "principal for deployment.\n",
+            log.getvalue())
+
+        # and verify no units assigned to service
+        service_state = yield self.service_state_manager.get_service_state("logging")
+        self.assertEqual(service_state.service_name, "logging")
+
+        units = yield service_state.get_unit_names()
+        self.assertEqual(units, [])
+
+    def test_deploy_legacy_keys_in_legacy_env(self):
+        yield self.client.delete("/constraints")
+
+        finished = self.setup_cli_reactor()
+        self.setup_exit(0)
+        self.mocker.replay()
+
+        main(["deploy", "--repository", self.unbundled_repo_path,
+              "local:sample", "beekeeper"])
+        yield finished
+
+        service_manager = ServiceStateManager(self.client)
+        yield service_manager.get_service_state("beekeeper")
+
+    @inlineCallbacks
+    def test_deploy_legacy_keys_in_fresh_env(self):
+        yield self.push_default_config()
+        local_config = {
+            "environments": {"firstenv": {
+                    "type": "dummy",
+                    "some-legacy-key": "blah",
+                    "default-series": "series"}}}
+        self.write_config(serializer.dump(local_config))
+        self.config.load()
+        finished = self.setup_cli_reactor()
+        self.setup_exit(0)
+        self.mocker.replay()
+        stderr = self.capture_logging()
+
+        main(["deploy", "--repository", self.unbundled_repo_path,
+              "local:sample", "beekeeper"])
+        yield finished
+
+        self.assertIn(
+            "Your environments.yaml contains deprecated keys",
+            stderr.getvalue())
+        service_manager = ServiceStateManager(self.client)
+        yield self.assertFailure(
+            service_manager.get_service_state("beekeeper"),
+            ServiceStateNotFound)
+
+    @inlineCallbacks
+    def test_deploy_constraints_in_legacy_env(self):
+        yield self.client.delete("/constraints")
+
+        finished = self.setup_cli_reactor()
+        self.setup_exit(0)
+        self.mocker.replay()
+        stderr = self.capture_logging()
+
+        main(["deploy", "--repository", self.unbundled_repo_path,
+              "local:sample", "beekeeper", "--constraints", "arch=i386"])
+        yield finished
+
+        self.assertIn(
+            "Constraints are not valid in legacy deployments.",
+            stderr.getvalue())
+        service_manager = ServiceStateManager(self.client)
+        yield self.assertFailure(
+            service_manager.get_service_state("beekeeper"),
+            ServiceStateNotFound)
