@@ -265,7 +265,7 @@ class ControlResolvedTest(
         self.mocker.replay()
         main(["resolved", "zebra/0"])
         yield finished
-        self.assertIn("Service 'zebra' was not found", self.stderr.getvalue())
+        self.assertIn("Service 'zebra' was not found", self.output.getvalue())
 
     @inlineCallbacks
     def test_resolved_unknown_unit(self):
@@ -361,7 +361,7 @@ class ControlResolvedTest(
             self.output.getvalue())
         self.assertIn(
             "Service unit 'mysql/0' is already marked as resolved.",
-            self.stderr.getvalue(), "")
+            self.output.getvalue(), "")
 
     @inlineCallbacks
     def test_resolved_relation_already_running(self):
