@@ -4,7 +4,7 @@ import json
 import logging
 import os
 from StringIO import StringIO
-import yaml
+
 
 from twisted.internet.defer import inlineCallbacks, returnValue
 
@@ -13,6 +13,7 @@ from juju.agents.machine import MachineAgent
 from juju.environment.environment import Environment
 from juju.control import status
 from juju.control import tests
+from juju.lib import serializer
 from juju.state.endpoint import RelationEndpoint
 from juju.state.environment import GlobalSettingsStateManager
 from juju.state.tests.test_service import ServiceStateManagerTestBase
@@ -23,7 +24,7 @@ from .common import ControlToolTest
 
 tests_path = os.path.dirname(inspect.getabsfile(tests))
 sample_path = os.path.join(tests_path, "sample_cluster.yaml")
-sample_cluster = yaml.load(open(sample_path, "r"))
+sample_cluster = serializer.load(open(sample_path, "r"))
 
 
 def dump_stringio(stringio, filename):
@@ -33,6 +34,17 @@ def dump_stringio(stringio, filename):
     fp.close()
 
 
+@inlineCallbacks
+def collect(scope, provider, client, log):
+    """Collect and return status info as dict"""
+    # provided for backwards compatibility with
+    # original API
+    # used only in testing
+    s = status.StatusCommand(client, provider, log)
+    state = yield s(scope)
+    returnValue(state)
+
+
 class StatusTestBase(ServiceStateManagerTestBase, ControlToolTest):
 
     # Status tests setup a large tree every time, make allowances for it.
@@ -46,17 +58,9 @@ class StatusTestBase(ServiceStateManagerTestBase, ControlToolTest):
         yield settings.set_provider_type("dummy")
         self.log = self.capture_logging()
 
-        config = {
-            "environments": {
-                "firstenv": {
-                    "type": "dummy",
-                    "admin-secret": "homer"}}}
-        self.write_config(yaml.dump(config))
-        self.config.load()
-
         self.environment = self.config.get_default()
         self.provider = self.environment.get_machine_provider()
-
+        self.machine_count = 0
         self.output = StringIO()
 
     @inlineCallbacks
@@ -84,6 +88,7 @@ class StatusTestBase(ServiceStateManagerTestBase, ControlToolTest):
             self,
             source_endpoint, source_units, source_states,
             dest_endpoint, dest_units, dest_states):
+
         relation_state, service_relation_states = \
             yield self.relation_state_manager.add_relation_state(
             *[source_endpoint, dest_endpoint])
@@ -94,11 +99,17 @@ class StatusTestBase(ServiceStateManagerTestBase, ControlToolTest):
             dest_relation_state, dest_units, dest_states)
 
     @inlineCallbacks
-    def add_unit(self, service, machine, with_agent=lambda _: True):
-        unit = yield service.add_unit_state()
-        yield unit.assign_to_machine(machine)
+    def add_unit(self, service, machine, with_agent=lambda _: True,
+                 units=None, container=None):
+        unit = yield service.add_unit_state(container=container)
+        self.assertTrue(machine or container)
+        if machine is not None:
+            yield unit.assign_to_machine(machine)
         if with_agent(unit.unit_name):
             yield unit.connect_agent()
+        if units is not None:
+            units.setdefault(service.service_name, []).append(unit)
+
         returnValue(unit)
 
     @inlineCallbacks
@@ -163,19 +174,23 @@ class StatusTestBase(ServiceStateManagerTestBase, ControlToolTest):
                 if fnmatch(name, pattern):
                     return False
             return True
-        wpu = yield self.add_unit(wordpress, m1, with_unit)
-        myu1 = yield self.add_unit(mysql, m2, with_unit)
-        myu2 = yield self.add_unit(mysql, m3, with_unit)
-        vu1 = yield self.add_unit(varnish, m4, with_unit)
-        vu2 = yield self.add_unit(varnish, m5, with_unit)
-        mc1 = yield self.add_unit(memcache, m6, with_unit)
-        mc2 = yield self.add_unit(memcache, m7, with_unit)
 
-         # add unit states to services and assign to machines
+
+        units = {}
+        wpu = yield self.add_unit(wordpress, m1, with_unit, units)
+        myu1 = yield self.add_unit(mysql, m2, with_unit, units)
+        myu2 = yield self.add_unit(mysql, m3, with_unit, units)
+        vu1 = yield self.add_unit(varnish, m4, with_unit, units)
+        vu2 = yield self.add_unit(varnish, m5, with_unit, units)
+        mc1 = yield self.add_unit(memcache, m6, with_unit, units)
+        mc2 = yield self.add_unit(memcache, m7, with_unit, units)
+        state["units"] = units
+
+        # add unit states to services and assign to machines
         # Set the lifecycle state and open ports, if any, for each unit state.
         yield self.set_unit_state(wpu, "started", [(80, "tcp"), (443, "tcp")])
         yield self.set_unit_state(myu1, "started")
-        yield self.set_unit_state(myu2, "stopped")
+        yield self.set_unit_state(myu2, "stop_error")
         yield self.set_unit_state(vu1, "started", [(80, "tcp")])
         yield self.set_unit_state(vu2, "started", [(80, "tcp")])
         yield self.set_unit_state(mc1, None)
@@ -225,18 +240,52 @@ class StatusTestBase(ServiceStateManagerTestBase, ControlToolTest):
 
 class StatusTest(StatusTestBase):
 
+    @inlineCallbacks
+    def add_provider_machine(self):
+        m = yield self.add_machine_state()
+        yield self.provider.start_machine(
+            {"machine-id": self.machine_count,
+             "dns-name": "steamcloud-%s.com" % self.machine_count})
+        m.set_instance_id(self.machine_count)
+        self.machine_count += 1
+        returnValue(m)
+
+    @inlineCallbacks
+    def test_status_provider_machine(self):
+        """Verify only one call to the provider for n machines.
+        """
+        yield self.add_provider_machine()
+        yield self.add_provider_machine()
+
+        mock_provider = self.mocker.patch(self.provider)
+        mock_provider.get_machines()
+        self.mocker.count(1)
+        self.mocker.passthrough()
+        self.mocker.replay()
+
+        state = yield collect(
+            None, mock_provider, self.client, None)
+        self.assertEqual(
+            state,
+            {'services': {},
+             'machines': {
+                 0: {'agent-state':
+                     'not-started',
+                     'instance-state':
+                     'unknown',
+                     'instance-id': 0,
+                     'dns-name': 'steamcloud-0.com'},
+                 1: {'agent-state': 'not-started',
+                     'instance-state': 'unknown',
+                     'instance-id': 1,
+                     'dns-name': 'steamcloud-1.com'}}})
+
     @inlineCallbacks
     def test_peer_relation(self):
         """Verify status works with peer relations.
         """
-        m1 = yield self.add_machine_state()
-        m2 = yield self.add_machine_state()
-        yield self.provider.start_machine({"machine-id": 0,
-                                           "dns-name": "steamcloud-1.com"})
-        yield self.provider.start_machine({"machine-id": 1,
-                                           "dns-name": "steamcloud-2.com"})
-        yield m1.set_instance_id(0)
-        yield m2.set_instance_id(1)
+        m1 = yield self.add_provider_machine()
+        m2 = yield self.add_provider_machine()
 
         riak = yield self.add_service_from_charm("riak")
         riak_u1 = yield self.add_unit(riak, m1)
@@ -248,25 +297,104 @@ class StatusTest(StatusTestBase):
             RelationEndpoint("riak", "peer", "ring", "peer"))
 
         riak_u1_relation = yield peer_rel.add_unit_state(riak_u1)
-        riak_u1_workflow = ZookeeperWorkflowState(self.client, riak_u1_relation)
+        riak_u1_workflow = ZookeeperWorkflowState(
+            self.client, riak_u1_relation)
         with (yield riak_u1_workflow.lock()):
             yield riak_u1_workflow.set_state("up")
         yield peer_rel.add_unit_state(riak_u2)
 
-        state = yield status.collect(
+        state = yield collect(
             ["riak"], self.provider, self.client, None)
         self.assertEqual(
             state["services"]["riak"],
             {"charm": "local:series/riak-7",
-             "relations": {"ring": "riak"},
+             "relations": {"ring": ["riak"]},
              "units": {"riak/0": {"machine": 0,
                                   "public-address": "riak-0.example.com",
-                                  "relations": {"ring": {"state": "up"}},
-                                  "state": "started"},
+                                  "agent-state": "started"},
                        "riak/1": {"machine": 1,
                                   "public-address": "riak-1.example.com",
-                                  "relations": {"ring": {"state": None}},
-                                  "state": "down"}}})
+                                  "agent-state": "down"}}})
+
+    @inlineCallbacks
+    def test_service_with_multiple_instances_of_named_relation(self):
+        m1 = yield self.add_provider_machine()
+        m2 = yield self.add_provider_machine()
+        m3 = yield self.add_provider_machine()
+
+        mysql = yield self.add_service_from_charm("mysql")
+        mysql_ep = RelationEndpoint(
+            "mysql", "client-server", "db", "server")
+        mysql_1 = yield self.add_unit(mysql, m1)
+
+        myblog = yield self.add_service_from_charm(
+            "myblog", charm_name="wordpress")
+        myblog_db_ep = RelationEndpoint(
+            "myblog", "client-server", "db", "client")
+        myblog_1 = yield self.add_unit(myblog, m2)
+
+        teamblog = yield self.add_service_from_charm(
+            "teamblog", charm_id=(yield myblog.get_charm_id()))
+
+        teamblog_db_ep = RelationEndpoint(
+            "teamblog", "client-server", "db", "client")
+        teamblog_1 = yield self.add_unit(teamblog, m3)
+
+        yield self.add_relation_with_relation_units(
+            mysql_ep, [mysql_1], ["up"],
+            myblog_db_ep, [myblog_1], ["up"])
+
+        yield self.add_relation_with_relation_units(
+            mysql_ep, [mysql_1], ["up"],
+            teamblog_db_ep, [teamblog_1], ["up"])
+
+        state = yield collect(None, self.provider, self.client, None)
+        self.assertEqual(
+            state["services"]["mysql"]["units"]["mysql/0"],
+            {"agent-state": "pending",
+             "machine": 0,
+             "public-address": None})
+        self.assertEqual(
+            state["services"]["mysql"]["relations"],
+            {"db": ["myblog", "teamblog"]})
+
+    @inlineCallbacks
+    def test_service_with_multiple_rels_to_same_endpoint(self):
+        m1 = yield self.add_provider_machine()
+        m2 = yield self.add_provider_machine()
+        mysql = yield self.add_service_from_charm("mysql")
+        mysql_ep = RelationEndpoint(
+            "mysql", "client-server", "db", "server")
+        mysql_1 = yield self.add_unit(mysql, m1)
+
+        myblog = yield self.add_service_from_charm(
+            "myblog", charm_name="funkyblog")
+        write_db_ep = RelationEndpoint(
+            "myblog", "client-server", "write-db", "client")
+        read_db_ep = RelationEndpoint(
+            "myblog", "client-server", "read-db", "client")
+        myblog_1 = yield self.add_unit(myblog, m2)
+
+        yield self.add_relation_with_relation_units(
+            mysql_ep, [mysql_1], ["down"],
+            write_db_ep, [myblog_1], ["up"])
+
+        yield self.add_relation_with_relation_units(
+            mysql_ep, [mysql_1], ["down"],
+            read_db_ep, [myblog_1], ["up"])
+
+        state = yield collect(None, self.provider, self.client, None)
+        # Even though there are two relations to this service we
+        # collapse to one the additional displays are redundant.
+        self.assertEqual(
+            state["services"]["mysql"]["relations"],
+            {"db": ["myblog"]})
+        self.assertEqual(
+            state["services"]["mysql"]["units"]["mysql/0"],
+            {"agent-state": "pending",
+             "machine": 0,
+             "relation-errors": {"db": ["myblog"]},
+             "public-address": None})
 
     @inlineCallbacks
     def test_collect(self):
@@ -284,7 +412,7 @@ class StatusTest(StatusTestBase):
         yield agent.start()
 
         # collect everything
-        state = yield status.collect(None, self.provider, self.client, None)
+        state = yield collect(None, self.provider, self.client, None)
         services = state["services"]
         self.assertIn("wordpress", services)
         self.assertIn("varnish", services)
@@ -299,76 +427,70 @@ class StatusTest(StatusTestBase):
                          {"instance-id": 0,
                           "instance-state": "unknown",
                           "dns-name": "steamcloud-1.com",
-                          "state": "running"})
+                          "agent-state": "running"})
 
         self.assertEqual(services["mysql"]["relations"],
-                         {"db": "wordpress"})
+                         {"db": ["wordpress"]})
 
         self.assertEqual(services["wordpress"]["relations"],
-                         {"cache": "memcache",
-                          "db": "mysql",
-                          "proxy": "varnish"})
+                         {"cache": ["memcache"],
+                          "db": ["mysql"],
+                          "proxy": ["varnish"]})
 
         self.assertEqual(
             services["varnish"],
             {"units":
                  {"varnish/1": {
                         "machine": 4,
-                        "state": "down",
+                        "agent-state": "down",
                         "open-ports": ["80/tcp"],
-                        "public-address": "varnish-1.example.com",
-                        "relations": {"proxy": {"state": "up"}}},
+                        "public-address": "varnish-1.example.com"},
                   "varnish/0": {
                         "machine": 3,
-                        "state": "started",
+                        "agent-state": "started",
                         "public-address": "varnish-0.example.com",
-                        "open-ports": ["80/tcp"],
-                        "relations": {"proxy": {"state": "up"}}}},
+                        "open-ports": ["80/tcp"]}},
              "exposed": True,
              "charm": "local:series/varnish-1",
-             "relations": {"proxy": "wordpress"}})
+             "relations": {"proxy": ["wordpress"]}})
 
         self.assertEqual(
             services["wordpress"],
             {"charm": "local:series/wordpress-3",
-             "relations":  {
-                    "cache": "memcache",
-                    "db": "mysql",
-                    "proxy": "varnish"},
+             "exposed": False,
+             "relations": {
+                    "cache": ["memcache"],
+                    "db": ["mysql"],
+                    "proxy": ["varnish"]},
              "units": {
                     "wordpress/0": {
                         "machine": 0,
                         "public-address": "wordpress-0.example.com",
-                        "relations": {
-                            "cache": {"state": "up"},
-                            "db": {"state": "up"},
-                            "proxy": {"state": "up"}},
-                        "state": "started"}}})
+                        "agent-state": "started"}}})
 
         self.assertEqual(
             services["memcache"],
             {"charm": "local:series/dummy-1",
-             "relations": {"cache": "wordpress"},
+             "relations": {"cache": ["wordpress"]},
              "units": {
-                    "memcache/0": {
-                        "machine": 5,
-                        "public-address": "memcache-0.example.com",
-                        "relations": {
-                            "cache": {"state": "up"}},
-                        "state": "pending"},
-                    "memcache/1": {
-                        "machine": 6,
-                        "public-address": "memcache-1.example.com",
-                        "relations": {
-                            "cache": {"state": "down"}},
-                        "state": "installed"}}})
+                 "memcache/0": {
+                     "machine": 5,
+                     "public-address": "memcache-0.example.com",
+                     "agent-state": "pending"},
+                 "memcache/1": {
+                     "machine": 6,
+                    "public-address": "memcache-1.example.com",
+                     "relation-errors": {
+                         "cache": ["wordpress"]},
+                     "agent-state": "installed"}}}
+            )
 
     @inlineCallbacks
     def test_collect_filtering(self):
         yield self.build_topology()
 
         # collect by service name
-        state = yield status.collect(
+        state = yield collect(
             ["wordpress"], self.provider, self.client, None)
         # Validate that only the expected service is present
         # in the state
@@ -376,13 +498,13 @@ class StatusTest(StatusTestBase):
         self.assertEqual(state["services"].keys(), ["wordpress"])
 
         # collect by unit name
-        state = yield status.collect(["*/0"], self.provider, self.client, None)
+        state = yield collect(["*/0"], self.provider, self.client, None)
         self.assertEqual(set(state["machines"].keys()), set([0, 1, 3, 5]))
         self.assertEqual(set(state["services"].keys()),
                          set(["memcache", "varnish", "mysql", "wordpress"]))
 
         # collect by unit name
-        state = yield status.collect(["*/1"], self.provider, self.client, None)
+        state = yield collect(["*/1"], self.provider, self.client, None)
         self.assertEqual(set(state["machines"].keys()), set([2, 4, 6]))
 
         # verify that only the proper units and services are present
@@ -390,44 +512,44 @@ class StatusTest(StatusTestBase):
             state["services"],
             {"memcache": {
                 "charm": "local:series/dummy-1",
-                    "relations": {"cache": "wordpress"},
+                    "relations": {"cache": ["wordpress"]},
                     "units": {
                         "memcache/1": {
                             "machine": 6,
-                            "state": "installed",
+                            "agent-state": "installed",
                             "public-address": "memcache-1.example.com",
-                            "relations": {"cache": {"state": "down"}}}}},
+                            "relation-errors": {"cache": ["wordpress"]}}}},
              "mysql": {
                     "exposed": True,
                     "charm": "local:series/mysql-1",
-                    "relations": {"db": "wordpress"},
+                    "relations": {"db": ["wordpress"]},
                     "units": {
                         "mysql/1": {
                             "machine": 2,
                             "public-address": "mysql-1.example.com",
                             "open-ports": [],
-                            "state": "stopped",
-                            "relations": {"db": {"state": "departed"}}}}},
+                            "agent-state": "stop-error",
+                            "relation-errors": {"db": ["wordpress"]}}}},
              "varnish": {
                     "exposed": True,
                     "charm": "local:series/varnish-1",
-                    "relations": {"proxy": "wordpress"},
+                    "relations": {"proxy": ["wordpress"]},
                     "units": {
                         "varnish/1": {
                             "machine": 4,
                             "public-address": "varnish-1.example.com",
                             "open-ports": ["80/tcp"],
-                            "state": "started",
-                            "relations": {"proxy": {"state": "up"}}}}}})
+                            "agent-state": "started",
+                            }}}})
 
         # filter a missing service
-        state = yield status.collect(
+        state = yield collect(
             ["cluehammer"], self.provider, self.client, None)
         self.assertEqual(set(state["machines"].keys()), set([]))
         self.assertEqual(set(state["services"].keys()), set([]))
 
         # filter a missing unit
-        state = yield status.collect(["*/7"], self.provider, self.client, None)
+        state = yield collect(["*/7"], self.provider, self.client, None)
         self.assertEqual(set(state["machines"].keys()), set([]))
         self.assertEqual(set(state["services"].keys()), set([]))
 
@@ -444,13 +566,13 @@ class StatusTest(StatusTestBase):
         yield unit.unassign_from_machine()
         yield unit.set_public_address(None)
         # test that the machine is in state information w/o assignment
-        state = yield status.collect(None, self.provider, self.client, None)
+        state = yield collect(None, self.provider, self.client, None)
         # verify that the unassigned machine appears in the state
         self.assertEqual(state["machines"][machine_id],
                          {"dns-name": "steamcloud-1.com",
                           "instance-id": 0,
                           "instance-state": "unknown",
-                          "state": "not-started"})
+                          "agent-state": "not-started"})
 
         # verify that we have a record of the unassigned service;
         # but note that unassigning this machine without removing the
@@ -461,11 +583,7 @@ class StatusTest(StatusTestBase):
             {"wordpress/0":
                  {"machine": None,
                   "public-address": None,
-                  "relations": {
-                        "cache": {"state": "up"},
-                        "db": {"state": "up"},
-                        "proxy": {"state": "up"}},
-                 "state": "started"}})
+                  "agent-state": "started"}})
 
     @inlineCallbacks
     def test_collect_with_removed_unit(self):
@@ -480,13 +598,13 @@ class StatusTest(StatusTestBase):
         yield wordpress.remove_unit_state(unit)
 
         # test that wordpress has no assigned service units
-        state = yield status.collect(None, self.provider, self.client, None)
+        state = yield collect(None, self.provider, self.client, None)
         self.assertEqual(
             state["services"]["wordpress"],
             {"charm": "local:series/wordpress-3",
-             "relations": {"cache": "memcache",
-                           "db": "mysql",
-                           "proxy": "varnish"},
+             "relations": {"cache": ["memcache"],
+                           "db": ["mysql"],
+                           "proxy": ["varnish"]},
              "units": {}})
 
         # but its machine is still available as reported by status
@@ -512,7 +630,7 @@ class StatusTest(StatusTestBase):
         yield wpu.assign_to_machine(m8)
 
         # test that we identify we don't have machine state
-        state = yield status.collect(
+        state = yield collect(
             None, self.provider, self.client, logging.getLogger())
         self.assertEqual(state["machines"][7]["instance-id"],
                          "pending")
@@ -525,7 +643,7 @@ class StatusTest(StatusTestBase):
 
         yield status.status(self.environment, [],
                             status.render_yaml, self.output, None)
-        state = yaml.load(self.output.getvalue())
+        state = serializer.yaml_load(self.output.getvalue())
 
         self.assertEqual(set(state["machines"].keys()),
                          set([0, 1, 2, 3, 4, 5, 6]))
@@ -539,7 +657,7 @@ class StatusTest(StatusTestBase):
         self.assertEqual(set(services["varnish"].keys()),
                          set(["exposed", "charm", "relations", "units"]))
         self.assertEqual(set(services["wordpress"].keys()),
-                         set(["charm", "relations", "units"]))
+                         set(["charm", "exposed", "relations", "units"]))
 
         for service in services.itervalues():
             self.assertGreaterEqual(  # may also include "exposed" key
@@ -551,18 +669,18 @@ class StatusTest(StatusTestBase):
                          {"instance-id": 0,
                           "instance-state": "unknown",
                           "dns-name": "steamcloud-1.com",
-                          "state": "down"})
+                          "agent-state": "down"})
 
         self.assertEqual(services["mysql"]["relations"],
-                         {"db": "wordpress"})
+                         {"db": ["wordpress"]})
 
         self.assertEqual(services["mysql"]["units"]["mysql/1"]["open-ports"],
                          [])
 
         self.assertEqual(services["wordpress"]["relations"],
-                         {"cache": "memcache",
-                          "db": "mysql",
-                          "proxy": "varnish"})
+                         {"cache": ["memcache"],
+                          "db": ["mysql"],
+                          "proxy": ["varnish"]})
 
     @inlineCallbacks
     def test_render_json(self):
@@ -586,7 +704,7 @@ class StatusTest(StatusTestBase):
         self.assertEqual(set(services["varnish"].keys()),
                          set(["exposed", "charm", "relations", "units"]))
         self.assertEqual(set(services["wordpress"].keys()),
-                         set(["charm", "relations", "units"]))
+                         set(["charm", "exposed", "relations", "units"]))
 
         for service in services.itervalues():
             self.assertTrue(service["charm"].startswith("local:series/"))
@@ -595,18 +713,18 @@ class StatusTest(StatusTestBase):
                          {"instance-id": 0,
                           "instance-state": "unknown",
                           "dns-name": "steamcloud-1.com",
-                          "state": "down"})
+                          "agent-state": "down"})
 
         self.assertEqual(services["mysql"]["relations"],
-                         {"db": "wordpress"})
+                         {"db": ["wordpress"]})
 
         self.assertEqual(services["mysql"]["units"]["mysql/1"]["open-ports"],
                          [])
 
         self.assertEqual(services["wordpress"]["relations"],
-                         {"cache": "memcache",
-                          "db": "mysql",
-                          "proxy": "varnish"})
+                         {"cache": ["memcache"],
+                          "db": ["mysql"],
+                          "proxy": ["varnish"]})
 
         self.assertEqual(
             services["varnish"],
@@ -617,16 +735,15 @@ class StatusTest(StatusTestBase):
                         "machine": 4,
                         "public-address": "varnish-1.example.com",
                         "open-ports": ["80/tcp"],
-                        "state": "started",
-                        "relations": {"proxy": {"state": "up"}}},
+                        "agent-state": "started"},
                      "varnish/0": {
                         "machine": 3,
                         "public-address": "varnish-0.example.com",
                         "open-ports": ["80/tcp"],
-                        "state": "started",
-                        "relations": {"proxy": {"state": "up"}}}},
+                        "agent-state": "started"},
+                     },
                      "charm": "local:series/varnish-1",
-                "relations": {"proxy": "wordpress"}})
+                "relations": {"proxy": ["wordpress"]}})
 
     @inlineCallbacks
     def test_render_dot(self):
@@ -652,13 +769,16 @@ class StatusTest(StatusTestBase):
         self.assertIn("mysql -> wordpress", result)
 
         # assert that properties were applied to a relationship
-        self.assertIn("wordpress -> varnish  [dir=none, label=proxy]",
-                      result)
+#        self.assertIn("wordpress -> varnish  [dir=none, "
+#                      "label=\"varnish:wordpress/proxy\"]",
+#                      result)
 
         # verify that the renderer picked up the DNS name of the
         # machines (and they are associated with the proper machine)
         self.assertIn(
-            '"mysql/0" [color="#DD4814", fontcolor="#ffffff", shape=box, style=filled, label=<mysql/0<br/><i>mysql-0.example.com</i>>]',
+            '"mysql/0" [color="#DD4814", fontcolor="#ffffff", '
+            "shape=box, style=filled, label=<mysql/0<br/><i>mysql-0."
+            "example.com</i>>]",
             result)
         self.assertIn(
             '"mysql/1" [color="#DD4814", fontcolor="#ffffff", shape=box, style=filled, label=<mysql/1<br/><i>mysql-1.example.com</i>>]',
@@ -704,7 +824,102 @@ class StatusTest(StatusTestBase):
                             self.output,
                             None)
 
-        #dump_stringio(self.output, "/tmp/ens.svg")
-
         # look for a hint the process completed.
         self.assertIn("</svg>", self.output.getvalue())
+
+    @inlineCallbacks
+    def test_subordinate_status_output(self):
+        state = yield self.build_topology()
+        # supplement status with additional subordinates
+        # add logging to mysql and wordpress
+        logging = yield self.add_service_from_charm("logging")
+
+        mysql_ep = RelationEndpoint("mysql", "client-server",
+                                    "juju-info", "server")
+        wordpress_db_ep = RelationEndpoint("wordpress", "client-server",
+                                           "juju-info", "server")
+        logging_ep = RelationEndpoint("logging", "client-server",
+                                      "juju-info", "client", "container")
+
+        my_log_rel, my_log_services = (
+            yield self.relation_state_manager.add_relation_state(
+                mysql_ep, logging_ep))
+        wp_log_rel, wp_log_services = (
+            yield self.relation_state_manager.add_relation_state(
+            wordpress_db_ep, logging_ep))
+
+        units = state["units"]
+        log_units = units.setdefault("logging", {})
+        wp1 = iter(units["wordpress"]).next()
+        mu1, mu2 = list(units["mysql"])
+
+        yield self.add_unit(logging, None, container=mu1, units=log_units)
+        yield self.add_unit(logging, None, container=wp1, units=log_units)
+        yield self.add_unit(logging, None, container=mu2, units=log_units)
+
+        self.mock_environment()
+        self.mocker.replay()
+
+        yield status.status(self.environment, [],
+                            status.render_yaml, self.output, None)
+
+        state = serializer.load(self.output.getvalue())
+
+        # verify our changes
+        log_state = state["services"]["logging"]
+
+        self.assertEqual(set(log_state["relations"]["juju-info"]),
+                         set(["mysql", "wordpress"]))
+        self.assertEqual(set(log_state["subordinate-to"]),
+                         set(["mysql", "wordpress"]))
+
+        wp_state = state["services"]["wordpress"]
+        self.assertEqual(wp_state["relations"]["juju-info"], ["logging"])
+        wp_subs = wp_state["units"]["wordpress/0"]["subordinates"]
+        logging_sub = wp_subs["logging/1"]
+        # this assertion verifies that we don't see keys we don't
+        # expect as well
+        self.assertEqual(logging_sub, {"agent-state": "pending"})
+
+    @inlineCallbacks
+    def test_subordinate_status_output_no_container(self):
+        state = yield self.build_topology()
+        # supplement status with additional subordinates
+        # add logging to mysql and wordpress
+        logging = yield self.add_service_from_charm("logging")
+
+        mysql_ep = RelationEndpoint("mysql", "client-server",
+                                    "juju-info", "server")
+        wordpress_db_ep = RelationEndpoint("wordpress", "client-server",
+                                           "juju-info", "server")
+        logging_ep = RelationEndpoint("logging", "client-server",
+                                      "juju-info", "client", "container")
+
+        my_log_rel, my_log_services = (
+            yield self.relation_state_manager.add_relation_state(
+                mysql_ep, logging_ep))
+        wp_log_rel, wp_log_services = (
+            yield self.relation_state_manager.add_relation_state(
+            wordpress_db_ep, logging_ep))
+
+        units = state["units"]
+        log_units = units.setdefault("logging", {})
+        wp1 = iter(units["wordpress"]).next()
+        mu1, mu2 = list(units["mysql"])
+
+        yield self.add_unit(logging, None, container=mu1, units=log_units)
+        yield self.add_unit(logging, None, container=wp1, units=log_units)
+        yield self.add_unit(logging, None, container=mu2, units=log_units)
+
+        # remove mysql/0
+        yield state["services"]["mysql"].remove_unit_state(mu1)
+
+        self.mock_environment()
+        self.mocker.replay()
+
+        yield status.status(self.environment, [],
+                            status.render_yaml, self.output, None)
+
+        output = serializer.load(self.output.getvalue())
+        self.assertNotIn(mu1.unit_name, output["services"]["mysql"]["units"])
+        self.assertIn(mu2.unit_name, output["services"]["mysql"]["units"])
