@@ -1,18 +1,20 @@
 import json
 import os
-from yaml import dump
 
 from twisted.internet.defer import inlineCallbacks, succeed
 
 from juju.charm.directory import CharmDirectory
-from juju.charm.repository import LocalCharmRepository
+from juju.charm.repository import LocalCharmRepository, CS_STORE_URL
 from juju.charm.tests.test_metadata import test_repository_path
 from juju.charm.url import CharmURL
 from juju.control import main
 from juju.errors import FileNotFound
 from juju.environment.environment import Environment
+from juju.lib.mocker import ANY
+from juju.lib.serializer import dump
 from juju.unit.workflow import UnitWorkflowState
 
+
 from .common import MachineControlToolTest
 
 
@@ -56,11 +58,7 @@ class ControlCharmUpgradeTest(
     @inlineCallbacks
     def setUp(self):
         yield super(ControlCharmUpgradeTest, self).setUp()
-        config = {
-            "environments": {"firstenv": {"type": "dummy"}}}
 
-        self.write_config(dump(config))
-        self.config.load()
         self.service_state1 = yield self.add_service_from_charm("mysql")
         self.service_unit1 = yield self.service_state1.add_unit_state()
 
@@ -162,6 +160,22 @@ class ControlCharmUpgradeTest(
         value = (yield self.service_unit1.get_upgrade_flag())
         self.assertTrue(value)
 
+    @inlineCallbacks
+    def test_force_charm_upgrade(self):
+        repository = self.increment_charm(self.charm)
+        with (yield self.unit1_workflow.lock()):
+            yield self.unit1_workflow.set_state("start_error")
+
+        finished = self.setup_cli_reactor()
+        self.setup_exit(0)
+        self.mocker.replay()
+
+        main(["upgrade-charm", "--repository", repository.path,
+              "--force", "mysql"])
+        yield finished
+        value = (yield self.service_unit1.get_upgrade_flag())
+        self.assertEqual(value, {'force': True})
+
     @inlineCallbacks
     def test_upgrade_charm_unknown_service(self):
         finished = self.setup_cli_reactor()
@@ -170,7 +184,7 @@ class ControlCharmUpgradeTest(
         main(["upgrade-charm", "--repository", self.makeDir(), "volcano"])
         yield finished
         self.assertIn(
-            "Service 'volcano' was not found", self.stderr.getvalue())
+            "Service 'volcano' was not found", self.output.getvalue())
 
     @inlineCallbacks
     def test_upgrade_charm_unknown_charm(self):
@@ -443,12 +457,6 @@ class RemoteUpgradeCharmTest(MachineControlToolTest):
     @inlineCallbacks
     def setUp(self):
         yield super(RemoteUpgradeCharmTest, self).setUp()
-        config = {
-            "environments": {"firstenv": {"type": "dummy"}}}
-
-        self.write_config(dump(config))
-        self.config.load()
-
         charm = CharmDirectory(os.path.join(
             test_repository_path, "series", "mysql"))
         self.charm_state_manager.add_charm_state(
@@ -475,8 +483,8 @@ class RemoteUpgradeCharmTest(MachineControlToolTest):
         self.setup_exit(0)
         getPage = self.mocker.replace("twisted.web.client.getPage")
         getPage(
-            "https://store.juju.ubuntu.com/"
-            "charm-info?charms=cs%3Aseries/mysql")
+            CS_STORE_URL + "/charm-info?charms=cs%3Aseries/mysql",
+            contextFactory=ANY)
         self.mocker.result(succeed(json.dumps(
             {"cs:series/mysql": {"revision": 1, "sha256": "whatever"}})))
         self.mocker.replay()
@@ -497,8 +505,9 @@ class RemoteUpgradeCharmTest(MachineControlToolTest):
         self.setup_exit(0)
 
         getPage = self.mocker.replace("twisted.web.client.getPage")
-        getPage(
-            "https://store.juju.ubuntu.com/charm-info?charms=cs%3Aseries/mysql")
+        getPage(CS_STORE_URL + "/charm-info?charms=cs%3Aseries/mysql",
+                contextFactory=ANY)
+
         self.mocker.result(succeed(json.dumps(
             {"cs:series/mysql": {"revision": 1, "sha256": "whatever"}})))
         self.mocker.replay()
