@@ -1,7 +1,6 @@
 import os
 
 from twisted.internet.defer import inlineCallbacks, returnValue
-from yaml import dump
 
 from juju.environment.tests.test_config import EnvironmentsConfigTestBase
 from juju.control.tests.common import ControlToolTest
@@ -10,6 +9,7 @@ from juju.control.utils import (
     expand_path, parse_passthrough_args, ParseError)
 from juju.environment.config import EnvironmentsConfig
 from juju.environment.errors import EnvironmentsConfigError
+from juju.lib.serializer import yaml_dump as dump
 from juju.lib.testing import TestCase
 from juju.state.errors import ServiceUnitStateMachineNotAssigned
 from juju.state.tests.test_service import ServiceStateManagerTestBase
