@@ -76,7 +76,7 @@ SCHEMA = KeyDict({
             optional=[
                 "access-key", "secret-key", "auth-url", "project-name",
                 "auth-mode", "region", "use-floating-ip",
-                "ssl-hostname-verification"]),
+                "ssl-hostname-verification", "default-instance-type"]),
         "openstack_s3": KeyDict({
             "control-bucket": String(),
             "admin-secret": String(),
@@ -96,21 +96,8 @@ SCHEMA = KeyDict({
             optional=[
                 "access-key", "secret-key", "combined-key", "auth-url",
                 "s3-uri", "project-name", "auth-mode", "region",
-                "use-floating-ip", "ssl-hostname-verification"]),
-        "orchestra": KeyDict({
-            "orchestra-server": String(),
-            "orchestra-user": String(),
-            "orchestra-pass": String(),
-            "admin-secret": String(),
-            "acquired-mgmt-class": String(),
-            "available-mgmt-class": String(),
-            "storage-url": String(),
-            "storage-user": String(),
-            "storage-pass": String(),
-            "placement": _EITHER_PLACEMENT,
-            "default-series": String()},
-            optional=[
-                "storage-url", "storage-user", "storage-pass", "placement"]),
+                "use-floating-ip", "ssl-hostname-verification",
+                "default-instance-type"]),
         "maas": KeyDict({
             "maas-server": String(),
             "maas-oauth": OAuthString(),
