@@ -26,20 +26,6 @@ environments:
     nofoo: 1
 """
 
-SAMPLE_ORCHESTRA = """
-environments:
-  sample:
-    type: orchestra
-    orchestra-server: somewhe.re
-    orchestra-user: user
-    orchestra-pass: pass
-    admin-secret: garden
-    acquired-mgmt-class: acquired
-    available-mgmt-class: available
-    storage-url: http://somewhereel.se
-    default-series: oneiric
-"""
-
 SAMPLE_MAAS = """
 environments:
   sample:
@@ -67,7 +53,6 @@ environments:
      admin-secret: sekret
      control-bucket: container
      default-image-id: 42
-     default-instance-type: m1-sample
      default-series: precise
 """
 
@@ -653,53 +638,6 @@ class EnvironmentsConfigTest(EnvironmentsConfigTestBase):
         provider = self.config.get_default().get_machine_provider()
         self.assertEqual(provider.config["ssl-hostname-verification"], True)
 
-    def test_orchestra_schema_requires(self):
-        requires = (
-            "type orchestra-server orchestra-user orchestra-pass "
-            "admin-secret acquired-mgmt-class available-mgmt-class "
-            "default-series").split()
-        for require in requires:
-            config = serializer.yaml_load(SAMPLE_ORCHESTRA)
-            del config["environments"]["sample"][require]
-            self.write_config(serializer.yaml_dump(config), other_path=True)
-
-            try:
-                self.config.load(self.other_path)
-            except EnvironmentsConfigError as error:
-                self.assertEquals(str(error),
-                                  "Environments configuration error: %s: "
-                                  "environments.sample.%s: "
-                                  "required value not found"
-                                  % (self.other_path, require))
-            else:
-                self.fail("Did not properly require %s when type == orchestra"
-                          % require)
-
-    def test_orchestra_respects_default_series(self):
-        config = serializer.yaml_load(SAMPLE_ORCHESTRA)
-        config["environments"]["sample"]["default-series"] = "magnificent"
-        self.write_config(serializer.yaml_dump(config), other_path=True)
-        self.config.load(self.other_path)
-
-        provider = self.config.get_default().get_machine_provider()
-        self.assertEqual(provider.config["default-series"], "magnificent")
-
-    def test_orchestra_verifies_placement(self):
-        config = serializer.yaml_load(SAMPLE_ORCHESTRA)
-        config["environments"]["sample"]["placement"] = "random"
-        self.write_config(serializer.yaml_dump(config), other_path=True)
-        e = self.assertRaises(
-            EnvironmentsConfigError, self.config.load, self.other_path)
-        self.assertIn("expected 'unassigned', got 'random'",
-                       str(e))
-
-        config["environments"]["sample"]["placement"] = "local"
-        self.write_config(serializer.yaml_dump(config), other_path=True)
-        self.config.load(self.other_path)
-
-        data = self.config.get_default().placement
-        self.assertEqual(data, "local")
-
     def test_maas_schema_requires(self):
         requires = "maas-server maas-oauth admin-secret default-series".split()
         for require in requires:
