@@ -1,4 +1,6 @@
 import argparse
+import base64
+import copy
 import json
 import logging
 import os
@@ -223,12 +225,25 @@ class CommandLineClient(object):
         stream.flush()
 
     def format_json(self, result, stream):
-        print >>stream, json.dumps(result)
+        encoded = copy.copy(result)
+        if isinstance(result, dict):
+            for k, v in result.iteritems():
+                # Workaround the fact that JSON does not work with str
+                # values that have high bytes and are not actually UTF-8
+                # encoded; workaround by firt testing whether it can be
+                # decoded as UTF-8, and if not, wrapping as Base64
+                # encoded.
+                if isinstance(v, str):
+                    try:
+                        v.decode("utf8")
+                    except UnicodeDecodeError:
+                        encoded[k] = base64.b64encode(v)
+        json.dump(encoded, stream)
 
     def format_smart(self, result, stream):
         if result is not None:
             charm_formatter = get_charm_formatter_from_env()
-            print >>stream, charm_formatter.format(result)
+            stream.write(charm_formatter.format_raw(result))
 
 
 def parse_log_level(level):
