@@ -54,7 +54,8 @@ from juju.errors import JujuError
 from juju.lib import serializer
 from juju.lib.format import get_charm_formatter, get_charm_formatter_from_env
 from juju.state.errors import (
-    InvalidRelationIdentity,  RelationStateNotFound, UnitRelationStateNotFound)
+    InvalidRelationIdentity,  RelationStateNotFound, UnitRelationStateNotFound,
+    RelationIdentNotFound)
 from juju.state.hook import RelationHookContext
 
 
@@ -92,7 +93,8 @@ class BaseCommand(amp.Command):
         UnitRelationStateNotFound: "UnitRelationStateNotFound",
         MustSpecifyRelationName: "MustSpecifyRelationName",
         InvalidRelationIdentity: "InvalidRelationIdentity",
-        RelationStateNotFound: "RelationStateNotFound"
+        RelationStateNotFound: "RelationStateNotFound",
+        RelationIdentNotFound: "RelationIdentNotFound"
         }
 
 
@@ -206,7 +208,7 @@ class UnitAgentServer(amp.AMP, object):
         if relation_id:
             yield self.factory.log(
                 logging.DEBUG, "Getting relation %s" % relation_id)
-            context = yield invoker.get_relation_hook_context(relation_id)
+            context = invoker.get_cached_relation_hook_context(relation_id)
         require_relation_context(context)
 
         try:
@@ -238,7 +240,7 @@ class UnitAgentServer(amp.AMP, object):
         if relation_id:
             yield self.factory.log(
                 logging.DEBUG, "Setting relation %s" % relation_id)
-            context = yield invoker.get_relation_hook_context(relation_id)
+            context = invoker.get_cached_relation_hook_context(relation_id)
         require_relation_context(context)
 
         for k, v in data.items():
@@ -256,8 +258,8 @@ class UnitAgentServer(amp.AMP, object):
         if relation_id:
             yield self.factory.log(
                 logging.DEBUG, "Listing relation members for %s" % relation_id)
-            context = yield self.factory.get_invoker(client_id).\
-                get_relation_hook_context(relation_id)
+            invoker = yield self.factory.get_invoker(client_id)
+            context = invoker.get_cached_relation_hook_context(relation_id)
         require_relation_context(context)
         members = yield context.get_members()
         defer.returnValue(dict(members=" ".join(members)))
@@ -265,7 +267,7 @@ class UnitAgentServer(amp.AMP, object):
     @RelationIdsCommand.responder
     @defer.inlineCallbacks
     def relation_ids(self, client_id, relation_name):
-        """Set values into state.hook.RelationHookContext.
+        """Get relation idents for this hook context.
 
         :client_id: hooks client id that is used to define a context
            for a consistent view of state.
