@@ -8,7 +8,6 @@ from argparse import ArgumentTypeError
 from contextlib import closing
 
 from twisted.internet.defer import inlineCallbacks, returnValue
-import yaml
 
 from juju.hooks.cli import (
     CommandLineClient, parse_log_level, parse_port_protocol)
@@ -423,39 +422,23 @@ class TestCli(TestCase):
         with closing(StringIO.StringIO()) as output:
             cli.format_smart(sample, output)
             self.assertEqual(output.getvalue(), formatted)
-            self.assertEqual(sample, yaml.safe_load(output.getvalue()))
 
     def test_format_smart_v2(self):
-        """Verifies smart format v2 writes correct YAML"""
+        """Verifies smart format v2 writes raw strings properly"""
         self.change_environment(_JUJU_CHARM_FORMAT="2")
 
         # For each case, verify actual output serialization along with
         # roundtripping through YAML
         self.assert_smart_output(None, "")  # No newline in output for None
-        self.assert_smart_output("", "''\n")
-        self.assert_smart_output("A string", "A string\n")
-        # Note: YAML uses b64 encoding for byte strings tagged by !!binary
+        self.assert_smart_output("", "")
+        self.assert_smart_output("A string", "A string")
         self.assert_smart_output(
-            "High bytes: \xCA\xFE",
-            "!!binary |\n    SGlnaCBieXRlczogyv4=\n")
-        self.assert_smart_output(u"", "''\n")
-        self.assert_smart_output(
-            u"A unicode string (but really ascii)",
-            "A unicode string (but really ascii)\n")
-        # Any non-ascii Unicode will use UTF-8 encoding
-        self.assert_smart_output(u"中文", "\xe4\xb8\xad\xe6\x96\x87\n")
-        self.assert_smart_output({}, "{}\n")
+            "High bytes: \xCA\xFE", "High bytes: \xca\xfe")
+        self.assert_smart_output("中文", "\xe4\xb8\xad\xe6\x96\x87")
         self.assert_smart_output(
             {u"public-address": u"ec2-1-2-3-4.compute-1.amazonaws.com",
              u"foo": u"bar",
              u"configured": True},
             ("configured: true\n"
              "foo: bar\n"
-             "public-address: ec2-1-2-3-4.compute-1.amazonaws.com\n"))
-        self.assert_smart_output(False, "false\n")
-        self.assert_smart_output(True, "true\n")
-        self.assert_smart_output(0.0, "0.0\n")
-        self.assert_smart_output(3.14159, "3.14159\n")
-        self.assert_smart_output(6.02214178e23, "6.02214178e+23\n")
-        self.assert_smart_output(0, "0\n")
-        self.assert_smart_output(42, "42\n")
+             "public-address: ec2-1-2-3-4.compute-1.amazonaws.com"))
