@@ -54,6 +54,9 @@ class MockServiceUnitState(object):
         self.ports = set()
         self.config = {}
 
+    def get_container(self):
+        return None
+
     def open_port(self, port, proto):
         self.ports.add((port, proto))
 
