@@ -24,7 +24,7 @@ from juju.lib.mocker import MATCH
 from juju.lib.twistutils import get_module_directory
 from juju.state import hook
 from juju.state.endpoint import RelationEndpoint
-from juju.state.errors import RelationStateNotFound
+from juju.state.errors import RelationIdentNotFound
 from juju.state.relation import RelationStateManager
 from juju.state.tests.test_relation import RelationTestBase
 
@@ -178,6 +178,7 @@ class InvokerTestBase(EnvironmentsConfigTestBase):
             "hooks")
         self.change_environment(
             PATH=get_cli_environ_path(test_hook_path, "/usr/bin", "/bin"),
+            JUJU_ENV_UUID="snowflake",
             JUJU_UNIT_NAME=local_unit,
             JUJU_REMOTE_UNIT=remote_unit)
 
@@ -268,8 +269,8 @@ class TestCompleteInvoker(InvokerTestBase, StatusTestBase):
             state["relations"]["mysql"][0])
         wpr = yield self.relation_state_manager.get_relations_for_service(
             state["services"]["wordpress"])
-        wpr = [r for r in wpr if r.internal_relation_id == \
-                 self.mysql_relation.internal_relation_id][0]
+        wpr = [r for r in wpr if r.internal_relation_id ==
+               self.mysql_relation.internal_relation_id][0]
         self.wordpress_relation = yield wpr.add_unit_state(
             state["relations"]["wordpress"][0])
 
@@ -289,9 +290,10 @@ class TestCompleteInvoker(InvokerTestBase, StatusTestBase):
         yield self.wordpress_relation.set_data({"hello": "world"})
 
         hook_log = self.capture_logging("hook")
-        exe = yield self.ua.get_invoker("db:42", "add", "mysql/0",
-                                  self.mysql_relation,
-                                  client_id="client_id")
+        exe = yield self.ua.get_invoker(
+            "db:42", "add", "mysql/0",
+            self.mysql_relation,
+            client_id="client_id")
 
         yield exe(self.create_hook(
             "relation-get", "--format=json - wordpress/0"))
@@ -542,6 +544,7 @@ class InvokerTest(RelationInvokerTestBase):
         # dummy charm (this is the default charm used when the
         # add_service method is use)
         self.assertEqual(env["_JUJU_CHARM_FORMAT"], "1")
+        self.assertEqual(env["JUJU_ENV_UUID"], "snowflake")
 
     @defer.inlineCallbacks
     def test_missing_hook(self):
@@ -1016,7 +1019,7 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         self.assertEqual(
             implied,
             self.ua.server_factory.get_invoker("client_id").\
-                get_relation_hook_context("db:0"))
+                get_cached_relation_hook_context("db:0"))
         self.assertEqual(
             set((yield implied.get_relation_idents("db"))),
             set(["db:0", "db:1"]))
@@ -1031,32 +1034,32 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         # Add another relation, verify it's not yet visible
         yield self.add_a_blog("wordpress3")
 
-        db0 = yield exe.get_relation_hook_context("db:0")
-        db1 = yield exe.get_relation_hook_context("db:1")
+        db0 = exe.get_cached_relation_hook_context("db:0")
+        db1 = exe.get_cached_relation_hook_context("db:1")
         self.assertEqual(db1.relation_ident, "db:1")
         self.assertEqual(
             set((yield db1.get_relation_idents("db"))),
             set(["db:0", "db:1"]))
         self.assertEqual(
             db1,
-            exe.get_relation_hook_context("db:1"))
+            exe.get_cached_relation_hook_context("db:1"))
 
         # Not yet visible relation
         self.assertRaises(
-            RelationStateNotFound,
-            exe.get_relation_hook_context, "db:2")
+            RelationIdentNotFound,
+            exe.get_cached_relation_hook_context, "db:2")
 
         # Nonexistent relation idents
         self.assertRaises(
-            RelationStateNotFound,
-            exe.get_relation_hook_context, "db:12345")
+            RelationIdentNotFound,
+            exe.get_cached_relation_hook_context, "db:12345")
 
         # Reread parent and child contexts
         exe = yield self.ua.get_invoker(
             "db:0", "add", "mysql/0", self.relation, client_id="client_id")
         db0 = yield exe.get_context()
-        db1 = yield exe.get_relation_hook_context("db:1")
-        db2 = yield exe.get_relation_hook_context("db:2")
+        db1 = exe.get_cached_relation_hook_context("db:1")
+        db2 = exe.get_cached_relation_hook_context("db:2")
 
         # Verify that any changes are written out; write directly here
         # using the relation contexts
@@ -1107,9 +1110,9 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
             set((yield exe.get_context().get_relation_idents("db"))),
             set(["db:0", "db:1", "db:2"]))
 
-        db0 = yield exe.get_relation_hook_context("db:0")
-        db1 = yield exe.get_relation_hook_context("db:1")
-        db2 = yield exe.get_relation_hook_context("db:2")
+        db0 = exe.get_cached_relation_hook_context("db:0")
+        db1 = exe.get_cached_relation_hook_context("db:1")
+        db2 = exe.get_cached_relation_hook_context("db:2")
 
         # Verify that any changes are written out; write directly here
         # using the relation contexts
@@ -1152,13 +1155,14 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         self.assertEqual(
             set((yield exe.get_context().get_relation_idents("db"))),
             set(["db:0", "db:2"]))
-        yield self.assertFailure((yield exe.get_relation_hook_context("db:1")),
-                                 RelationStateNotFound)
+        yield self.assertRaises(
+            RelationIdentNotFound,
+            exe.get_cached_relation_hook_context, "db:1")
 
     @defer.inlineCallbacks
     def test_relation_ids(self):
         """Verify `relation-ids` command returns ids separated by newlines."""
-        yield self.add_a_blog("wordpress2")
+
         hook_log = self.capture_logging("hook")
 
         # Invoker will be in the context of the mysql/0 service unit
@@ -1166,6 +1170,10 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
             "db:0", "add", "mysql/0", self.relation,
             client_id="client_id")
 
+        # Invoker has already been started, verify cache coherence by
+        # adding another relation.
+        yield self.add_a_blog("wordpress2")
+
         # Then verify the hook lists the relation ids corresponding to
         # the relation name `db`
         hook = self.create_hook("relation-ids", "db")
@@ -1174,12 +1182,12 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         yield exe.ended
         # Smart formtting outputs one id per line
         self.assertEqual(
-            hook_log.getvalue(), "db:0\ndb:1\n\n")
+            hook_log.getvalue(), "db:0\n\n")
         # But newlines are just whitespace to the shell or to Python,
         # so they can be split accordingly, adhering to the letter of
         # the spec
         self.assertEqual(
-            hook_log.getvalue().split(), ["db:0", "db:1"])
+            hook_log.getvalue().split(), ["db:0"])
 
     @defer.inlineCallbacks
     def test_relation_ids_json_format(self):
@@ -1288,11 +1296,11 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         self.assertEqual(result, 0)
         yield exe.ended
 
-        db1 = exe.get_relation_hook_context("db:1")
+        db1 = exe.get_cached_relation_hook_context("db:1")
         yield self.assert_zk_data(db1, {
-                "a": "42",
-                "b": "xyz",
-                "private-address": "mysql-0.example.com"})
+            "a": "42",
+            "b": "xyz",
+            "private-address": "mysql-0.example.com"})
         self.assertLogLines(
             self.log.getvalue(),
             ["Cached relation hook contexts on 'db:0': ['db:1']",
@@ -1319,7 +1327,7 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         self.assertEqual(result, 0)
         yield exe.ended
         self.assert_file(args["stdout"], "")
-        self.assert_file(args["stderr"], "Relation not found\n")
+        self.assert_file(args["stderr"], "Relation not found for - db:12345\n")
         self.assertEqual(
             hook_log.getvalue(),
             "Cached relation hook contexts on 'db:0': ['db:1']\n"
@@ -1363,7 +1371,7 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
             client_id="client_id")
 
         # First set through the context
-        db1 = exe.get_relation_hook_context("db:1")
+        db1 = exe.get_cached_relation_hook_context("db:1")
         yield db1.set({"name": "whiterabbit"})
 
         # Then get from db:1
@@ -1396,7 +1404,7 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         self.assertEqual(result, 0)
         yield exe.ended
         self.assert_file(args["settings"], "")
-        self.assert_file(args["stderr"], "Relation not found\n")
+        self.assert_file(args["stderr"], "Relation not found for - db:12345\n")
         self.assertEqual(
             hook_log.getvalue(),
             "Cached relation hook contexts on 'db:0': ['db:1']\n"
@@ -1489,7 +1497,7 @@ class ChildRelationHookContextsTest(RelationInvokerTestBase):
         self.assertEqual(result, 0)
         yield exe.ended
         self.assert_file(args["stdout"], "")
-        self.assert_file(args["stderr"], "Relation not found\n")
+        self.assert_file(args["stderr"], "Relation not found for - db:12345\n")
         self.assertEqual(
             hook_log.getvalue(),
             "Cached relation hook contexts on 'db:0': ['db:1']\n"
