@@ -26,9 +26,11 @@ JUJU_PUBLIC_KEY: An SSH public key used by the ubuntu account for
 
 """
 
+DEVTMPFS_LINE = """devtmpfs dev devtmpfs mode=0755,nosuid 0 0"""
+
 # Used to specify the name of the default LXC template used
 # for container creation
-DEFAULT_TEMPLATE = "ubuntu"
+DEFAULT_TEMPLATE = "ubuntu-cloud"
 
 
 class LXCError(JujuError):
@@ -48,15 +50,27 @@ def _cmd(args):
 
 
 # Wrapped lxc cli primitives
-def _lxc_create(container_name, template, release, config_file=None):
+def _lxc_create(container_name,
+                template,
+                release,
+                cloud_init_file=None,
+                auth_key=None,
+                release_stream=None):
     # the -- argument indicates the last parameters are passed
     # to the template and not handled by lxc-create
     args = ["sudo", "lxc-create",
             "-n", container_name,
             "-t", template,
-            "-f", config_file,
             "--",
+            "--debug",  # Debug erors / set -x
+            "--hostid", container_name,
             "-r", release]
+    if cloud_init_file:
+        args.extend(("--userdata", cloud_init_file))
+    if auth_key:
+        args.extend(("-S", auth_key))
+    if release_stream:
+        args.extend(("-s", release_stream))
     return _cmd(args)
 
 
@@ -147,34 +161,42 @@ def get_containers(prefix):
     returnValue(containers)
 
 
+def ensure_devtmpfs_fstab(container_home):
+    """ Workaround for bug in older LXC - We need to force mounting devtmpfs
+        if it is not already in the rootfs, before starting.
+    """
+    rootfs = os.path.join(container_home, 'rootfs')
+    devpts = os.path.join(rootfs, 'dev', 'pts')
+    if not os.path.exists(devpts):
+        fstab_path = os.path.join(container_home, 'fstab')
+        if os.path.exists(fstab_path):
+            with open(fstab_path) as fstab:
+                for line in fstab:
+                    if line.startswith('devtmpfs'):
+                        # Line already there, we are done
+                        return
+            mode = 'a'
+        else:
+            mode = 'w'
+        with open(fstab_path, mode) as fstab:
+            print >>fstab, DEVTMPFS_LINE
+
+
 class LXCContainer(object):
     def __init__(self,
                  container_name,
-                 public_key,
                  series,
-                 origin,
-                 origin_source=None,
-                 network_name="virbr0",
-                 customize_script=None,
+                 cloud_init=None,
                  debug_log=None,
                  console_log=None,
-                 customize_log=None):
+                 release_stream="released"):
         """Create an LXCContainer
 
         :param container_name: should be unique within the system
 
-        :param public_key: SSH public key
-
         :param series: distro release series (oneiric, precise, etc)
 
-        :param origin: distro|ppa|branch
-
-        :param origin_source: when origin is branch supply a valid bzr branch
-
-        :param network_name: name of network link
-
-        :param customize_script: script used inside container to make
-        it juju ready
+        :param cloud_init: full string of cloud-init userdata
 
         See :data CONFIG_OPTIONS_DOC: explain how these values map
         into the container in more detail.
@@ -182,22 +204,17 @@ class LXCContainer(object):
         self.container_name = container_name
         self.debug_log = debug_log
         self.console_log = console_log
-        self.customize_log = customize_log
-        if customize_script is None:
-            customize_script = os.path.join(DATA_PATH,
-                                            "juju-create")
-        self.customize_script = customize_script
-        validate_path(self.customize_script)
-
-        self.public_key = public_key
-        self.origin = origin
-        self.source = origin_source
+        self.cloud_init = cloud_init
         self.series = series
-        self.network_name = network_name
+        self.release_stream = release_stream
+
+    @property
+    def container_home(self):
+        return "/var/lib/lxc/%s" % self.container_name
 
     @property
     def rootfs(self):
-        return "/var/lib/lxc/%s/rootfs/" % self.container_name
+        return "%s/rootfs/" % self.container_home
 
     def _p(self, path):
         if path[0] == "/":
@@ -205,9 +222,8 @@ class LXCContainer(object):
         return os.path.join(self.rootfs, path)
 
     def is_constructed(self):
-        """Does the lxc image exist and has juju-create been run in it."""
-        return os.path.exists(self.rootfs) and \
-            os.path.exists(self._p("/etc/juju/juju.conf"))
+        """Does the lxc image exist """
+        return os.path.exists(self.rootfs)
 
     @inlineCallbacks
     def is_running(self):
@@ -215,48 +231,6 @@ class LXCContainer(object):
         state = yield get_containers(None)
         returnValue(state.get(self.container_name))
 
-    def _make_lxc_config(self, network_name):
-        lxc_config = os.path.join(DATA_PATH, "lxc.conf")
-        with open(lxc_config, "r") as fh:
-            template = fh.read()
-            fd, output_fn = tempfile.mkstemp(suffix=".conf")
-            output_config = open(output_fn, "w")
-            output_config.write(template % {"network_name": network_name})
-            output_config.close()
-
-            validate_path(output_fn)
-            return output_fn
-
-    def _customize_container(self):
-
-        config_dir = self._p("etc/juju")
-        if not os.path.exists(config_dir):
-            _cmd(["sudo", "mkdir", config_dir])
-
-        _, fn = tempfile.mkstemp()
-
-        def escape(key, value):
-            return "%s=%s" % (key, pipes.quote(value))
-
-        with open(fn, "w") as fp:
-            target = self._p("etc/juju/juju.conf")
-            docstring = "\n#".join([""] + CONTAINER_OPTIONS_DOC.splitlines())
-            print >>fp, docstring
-
-            print >>fp, escape("JUJU_CONTAINER_NAME", self.container_name)
-            print >>fp, escape("JUJU_PUBLIC_KEY", self.public_key)
-            print >>fp, escape("JUJU_ORIGIN", self.origin)
-            print >>fp, escape("JUJU_SERIES", self.series)
-            if self.source:
-                print >>fp, escape("JUJU_SOURCE", self.source)
-
-        _cmd(["sudo", "mv", fn, target])
-        _, output = _customize_container(self.customize_script, self.rootfs)
-
-        if self.customize_log:
-            with open(self.customize_log, "w") as fh:
-                fh.write(output)
-
     def execute(self, args):
         if not isinstance(args, (list, tuple)):
             args = [args, ]
@@ -266,15 +240,22 @@ class LXCContainer(object):
 
     def _create_wait(self):
         """Create the container synchronously."""
-        if not self.is_constructed():
-            lxc_config = self._make_lxc_config(self.network_name)
-            _lxc_create(
-                self.container_name,
-                template=DEFAULT_TEMPLATE,
-                release=self.series,
-                config_file=lxc_config)
-            os.unlink(lxc_config)
-            self._customize_container()
+        if self.is_constructed():
+            return
+
+        with tempfile.NamedTemporaryFile() as fh:
+            if self.cloud_init:
+                fh.write(self.cloud_init.render())
+                cloud_init_file = fh.name
+            else:
+                cloud_init_file = None
+            fh.flush()
+            _lxc_create(self.container_name,
+                        template=DEFAULT_TEMPLATE,
+                        cloud_init_file=cloud_init_file,
+                        release=self.series)
+
+        ensure_devtmpfs_fstab(self.container_home)
 
     @inlineCallbacks
     def create(self):
@@ -292,14 +273,11 @@ class LXCContainer(object):
                            "that hasn't been had create() called")
 
         container = LXCContainer(container_name,
-                                 public_key=self.public_key,
-                                 origin=self.origin,
-                                 origin_source=self.source,
-                                 debug_log=self.debug_log,
-                                 console_log=self.console_log,
-                                 customize_script=self.customize_script,
-                                 network_name=self.network_name,
-                                 series=self.series)
+                 series=self.series,
+                 cloud_init=self.cloud_init,
+                 debug_log=self.debug_log,
+                 console_log=self.console_log,
+                 release_stream=self.release_stream)
 
         if not container.is_constructed():
             _lxc_clone(self.container_name, container_name)
