@@ -1,5 +1,5 @@
 import os
-import tempfile
+import inspect
 
 from twisted.internet.defer import inlineCallbacks
 from twisted.internet.threads import deferToThread
@@ -7,8 +7,11 @@ from twisted.internet.threads import deferToThread
 from juju.lib.lxc import (_lxc_start, _lxc_stop, _lxc_create,
                           _lxc_wait, _lxc_ls, _lxc_destroy,
                           LXCContainer, get_containers, LXCError,
-                          DEFAULT_TEMPLATE)
+                          DEFAULT_TEMPLATE, ensure_devtmpfs_fstab,
+                          tests)
 from juju.lib.testing import TestCase
+from juju.providers.common.cloudinit import CloudInit
+from juju.machine import ProviderMachine
 
 
 def skip_sudo_tests():
@@ -23,10 +26,7 @@ def uses_sudo(f):
     f.skip = skip_sudo_tests()
     return f
 
-
-DATA_PATH = os.path.abspath(
-    os.path.join(os.path.dirname(__file__), "..", "data"))
-
+DATA_DIR = os.path.join(os.path.dirname(inspect.getabsfile(tests)), "data")
 
 DEFAULT_SERIES = "precise"
 DEFAULT_CONTAINER = "lxc_test"
@@ -37,27 +37,13 @@ class LXCTest(TestCase):
     timeout = 240
 
     def setUp(self):
-        self.config = self.make_config()
-
-        @self.addCleanup
-        def remove_config():
-            if os.path.exists(self.config):
-                os.unlink(self.config)
+        self.cloud_init_file = self.mktemp()
+        with open(self.cloud_init_file, 'w') as cloud_init:
+            cloud_init.write("# cloud-init\n")
 
     def tearDown(self):
         self.clean_container(DEFAULT_CONTAINER)
 
-    def make_config(self, network_name="virbr0"):
-        lxc_config = os.path.join(DATA_PATH, "lxc.conf")
-        template = open(lxc_config, "r").read()
-
-        fd, output_fn = tempfile.mkstemp(suffix=".conf")
-        output_config = open(output_fn, "w")
-        output_config.write(template % {"network_name": network_name})
-        output_config.close()
-
-        return output_fn
-
     def clean_container(self, container_name):
         if os.path.exists("/var/lib/lxc/%s" % container_name):
             _lxc_stop(container_name)
@@ -67,7 +53,7 @@ class LXCTest(TestCase):
         self.addCleanup(self.clean_container, DEFAULT_CONTAINER)
 
         _lxc_create(DEFAULT_CONTAINER, DEFAULT_TEMPLATE, DEFAULT_SERIES,
-                    config_file=self.config)
+                self.cloud_init_file)
 
         # verify we can find the container
         output = _lxc_ls()
@@ -82,7 +68,7 @@ class LXCTest(TestCase):
         self.addCleanup(self.clean_container, DEFAULT_CONTAINER)
 
         _lxc_create(DEFAULT_CONTAINER, DEFAULT_TEMPLATE, DEFAULT_SERIES,
-                    config_file=self.config)
+                self.cloud_init_file)
         _lxc_start(DEFAULT_CONTAINER)
         _lxc_stop(DEFAULT_CONTAINER)
 
@@ -91,18 +77,14 @@ class LXCTest(TestCase):
         self.addCleanup(self.clean_container, DEFAULT_CONTAINER)
         yield deferToThread(
             _lxc_create, DEFAULT_CONTAINER, DEFAULT_TEMPLATE, DEFAULT_SERIES,
-            config_file=self.config)
+            self.cloud_init_file)
         yield deferToThread(_lxc_start, DEFAULT_CONTAINER)
 
     @inlineCallbacks
     def test_lxc_container(self):
         self.addCleanup(self.clean_container, DEFAULT_CONTAINER)
-        customize_log = self.makeFile()
         c = LXCContainer(DEFAULT_CONTAINER,
-                         "dsa...",
-                         "precise",
-                         "ppa",
-                         customize_log=customize_log)
+                         "precise")
         running = yield c.is_running()
         self.assertFalse(running)
         self.assertFalse(c.is_constructed())
@@ -128,7 +110,7 @@ class LXCTest(TestCase):
         self.assertTrue(os.path.exists(c.rootfs))
         self.assertTrue(c.is_constructed())
 
-        self.verify_container(c, "dsa...", "precise", "ppa")
+        self.verify_container(c, series="precise")
 
         # Verify that we are in containers
         containers = yield get_containers(None)
@@ -142,9 +124,6 @@ class LXCTest(TestCase):
         containers = yield get_containers(None)
         self.assertNotIn(DEFAULT_CONTAINER, containers)
 
-        # Verify the customize log file.
-        self.assertTrue(os.path.exists(customize_log))
-
         # and its gone
         output = _lxc_ls()
         self.assertNotIn(DEFAULT_CONTAINER, output)
@@ -154,7 +133,7 @@ class LXCTest(TestCase):
         self.addCleanup(self.clean_container, DEFAULT_CONTAINER)
 
         _lxc_create(DEFAULT_CONTAINER, DEFAULT_TEMPLATE, DEFAULT_SERIES,
-                    config_file=self.config)
+                self.cloud_init_file)
 
         _lxc_start(DEFAULT_CONTAINER)
 
@@ -174,10 +153,14 @@ class LXCTest(TestCase):
         self.addCleanup(self.clean_container, DEFAULT_CONTAINER)
         self.addCleanup(self.clean_container, DEFAULT_CONTAINER + "_child")
 
+        cloud_init = CloudInit()
+        cloud_init.add_ssh_key('dsa...')
+        cloud_init.set_provider_type('local')
+        cloud_init.set_zookeeper_machines([ProviderMachine('localhost','localhost','localhost')])
+
         master_container = LXCContainer(DEFAULT_CONTAINER,
-                                        origin="ppa",
-                                        public_key="dsa...",
-                                        series="oneiric")
+                                        series="precise",
+                                        cloud_init=cloud_init)
 
         # verify that we cannot clone an unconstructed container
         failure = master_container.clone("test_lxc_fail")
@@ -201,7 +184,7 @@ class LXCTest(TestCase):
         output = _lxc_ls()
         self.assertIn(DEFAULT_CONTAINER, output)
 
-        self.verify_container(c, "dsa...", "oneiric", "ppa")
+        self.verify_container(c, series="precise", cloud_init=cloud_init.render())
 
         # verify that we are in containers
         containers = yield get_containers(None)
@@ -221,7 +204,20 @@ class LXCTest(TestCase):
 
         yield master_container.destroy()
 
-    def verify_container(self, c, public_key, series, origin):
+    def test_create_wait(self):
+        self.addCleanup(self.clean_container, DEFAULT_CONTAINER)
+
+        cinit = CloudInit()
+        cinit.add_ssh_key('dsa...')
+        cinit.set_provider_type('local')
+        cinit.set_zookeeper_machines([ProviderMachine('localhost','localhost','localhost')])
+
+        c = LXCContainer(DEFAULT_CONTAINER, series="precise", cloud_init=cinit)
+        c._create_wait()
+        self.verify_container(c, "precise", cinit.render())
+
+
+    def verify_container(self, c, series, cloud_init=None):
         """Verify properties of an LXCContainer"""
 
         def p(path):
@@ -240,63 +236,65 @@ class LXCTest(TestCase):
             return rc
 
         # basic path checks
-        for path in ("etc/juju", "var/lib/juju"):
-            self.assertTrue(os.path.exists(p(path)))
-
-        # verify packages we depend on are installed
-        for pkg in ("resolvconf", "sudo"):
-            self.assertEqual(run(["dpkg-query", "-s", pkg]), 0)
+        self.assertTrue(os.path.exists(p('var/lib/cloud/seed/nocloud-net')))
 
         # ubuntu user
         self.assertEqual(run(["id", "ubuntu"]), 0)
 
-        # public key checks
-        pub = sudo_get("home/ubuntu/.ssh/authorized_keys")
-        self.assertEqual(pub.strip(), public_key)
-
-        # sudoers access
-        sudoers = sudo_get("etc/sudoers.d/lxc")
-        self.assertIn("ubuntu ALL=(ALL:ALL) NOPASSWD: ALL", sudoers)
-
-        # ssh disabled password auth
-        sshd_config = sudo_get("etc/ssh/sshd_config")
-        self.assertIn("PasswordAuthentication no", sshd_config)
-
-        # hostname
-        self.assertEqual(c.container_name, sudo_get("etc/hostname").strip())
-        # the lxc-clone command provides a different ordering here
-        # we'd have to run customize_constainer again which removes
-        # some of the point of the clone support to repair this.
-        # droppping assertion for now and replacing with a lax one
-        #XXX::: self.assertIn("127.0.0.1 %s localhost" % c.container_name,
-        #XXX    sudo_get("etc/hosts"))
-
-        self.assertIn(c.container_name, sudo_get("etc/hosts"))
-
-        # nameserver
-        resolv_conf = sudo_get("etc/resolvconf/resolv.conf.d/base")
-        self.assertIn("nameserver 192.168.122.1", resolv_conf)
-
-        # verify apt-cacher
-        apt_proxy = sudo_get("/etc/apt/apt.conf.d/02juju-apt-proxy")
-        self.assertIn('Acquire::http { Proxy "http://192.168.122.1:3142"; };',
-                      apt_proxy)
-        self.assertIn('Acquire::https { Proxy "false"; };',
-                      apt_proxy)
-
         # Verify the container release series.
         with open(os.path.join(c.rootfs, "etc", "lsb-release")) as fh:
             lsb_info = fh.read()
         self.assertIn(series, lsb_info)
 
-        # check basic juju installation
-        # these could be more through
-        if origin == "ppa":
-            self.assertEqual(0, run(["dpkg-query", "-s", "juju"]))
-        elif origin == "distro":
-            self.assertEqual(0, run(["dpkg-query", "-s", "juju"]))
-        elif origin == "branch":
-            # package isn't installed
-            self.assertEqual(1, run(["dpkg-query", "-s", "juju"]))
-            # but the branch is checked out
-            self.asssertTrue(os.path.exists(p("usr/lib/juju/juju")))
+
+class LXCUtilTest(TestCase):
+    def test_ensure_devtmpfs_fstab(self):
+        without_dev_pts = self.makeDir()
+        os.mkdir(os.path.join(without_dev_pts, 'rootfs'))
+        ensure_devtmpfs_fstab(without_dev_pts)
+        fstab_path = os.path.join(without_dev_pts, 'fstab')
+        if os.path.exists(fstab_path):
+            with open(fstab_path) as fstab:
+                found = False
+                for line in fstab:
+                    if line.startswith('devtmpfs'):
+                        found = True
+                        break
+                self.assertTrue(found)
+        else:
+            self.fail('fstab missing')
+
+    def test_ensure_devtmpfs_fstab_pts_exists(self):
+        with_dev_pts = self.makeDir()
+        os.mkdir(os.path.join(with_dev_pts, 'rootfs'))
+        os.mkdir(os.path.join(with_dev_pts, 'rootfs', 'dev'))
+        os.mkdir(os.path.join(with_dev_pts, 'rootfs', 'dev', 'pts'))
+        ensure_devtmpfs_fstab(with_dev_pts)
+        fstab_path = os.path.join(with_dev_pts, 'fstab')
+        if os.path.exists(fstab_path):
+            self.fail('fstab created but pts already there')
+
+    def test_ensure_devtmpfs_fstab_line_added(self):
+        self._test_devtmpfs_line('sample_fstab')
+
+    def test_ensure_devtmpfs_fstab_line_onlyone(self):
+        self._test_devtmpfs_line('sample_fstab_withdevtmpfs')
+
+    def _test_devtmpfs_line(self, datafile):
+        without_dev_pts = self.makeDir()
+        os.mkdir(os.path.join(without_dev_pts, 'rootfs'))
+        fstab_path = os.path.join(without_dev_pts, 'fstab')
+        with open(fstab_path, 'w') as fstab:
+            source = os.path.join(DATA_DIR, datafile)
+            with open(source) as src:
+                fstab.write(src.read())
+        ensure_devtmpfs_fstab(without_dev_pts)
+        if os.path.exists(fstab_path):
+            with open(fstab_path) as fstab:
+                count = 0
+                for line in fstab:
+                    if line.startswith('devtmpfs'):
+                        count += 1
+                self.assertEquals(count, 1)
+        else:
+            self.fail('fstab missing')
