@@ -11,6 +11,9 @@ import subprocess
 
 from twisted.internet.threads import deferToThread
 
+CLIENT_SESSION_TIMEOUT = 30000
+TICK_TIME = int(CLIENT_SESSION_TIMEOUT / 2)
+MAX_SESSION_TIMEOUT = CLIENT_SESSION_TIMEOUT * 2
 
 zookeeper_script_template = """\
 #!/bin/bash
@@ -39,6 +42,8 @@ tickTime=2000
 dataDir=%s
 clientPort=%s
 maxClientCnxns=500
+minSessionTimeout=%d
+maxSessionTimeout=%d
 """
 
 
@@ -51,7 +56,10 @@ class Zookeeper(object):
 
     def __init__(self, run_dir, port=None, host=None,
                  zk_location="system", user=None, group=None,
-                 use_deferred=True, fsync=True):
+                 use_deferred=True,
+                 min_session_timeout=CLIENT_SESSION_TIMEOUT,
+                 max_session_timeout=MAX_SESSION_TIMEOUT,
+                 fsync=True):
         """
         :param run_dir: Directory to store all zookeeper instance related data.
         :param port: The port zookeeper should run on.
@@ -72,6 +80,8 @@ class Zookeeper(object):
         self._user = user
         self._group = group
         self._zk_location = zk_location
+        self._min_session_time = min_session_timeout
+        self._max_session_time = max_session_timeout
         self._use_deferred = use_deferred
         self.fsync = fsync
 
@@ -198,9 +208,12 @@ class Zookeeper(object):
             config.write(log4j_properties)
 
         with open(zk_vars["config_path"], "w") as config:
-            conf = zookeeper_conf_template % (zk_vars["data_dir"], self._port)
-            if self.fsync:
-                conf += "\nfsync=no\n"
+            conf = zookeeper_conf_template % (
+                zk_vars["data_dir"], self._port,
+                self._min_session_time,
+                self._max_session_time)
+            if not self.fsync:
+                conf += "\nforceSync=no\n"
             config.write(conf)
             if self._host:
                 config.write("\nclientPortAddress=%s" % self._host)
