@@ -8,7 +8,7 @@ from twisted.internet.defer import inlineCallbacks, returnValue
 
 from juju.charm.bundle import CharmBundle
 from juju.errors import ServiceError
-from juju.lib.lxc import LXCContainer, get_containers, LXCError
+from juju.lib.lxc import LXCContainer, get_containers
 from juju.lib.twistutils import get_module_directory
 from juju.lib.upstart import UpstartService
 from juju.providers.common.cloudinit import CloudInit
@@ -27,12 +27,17 @@ def get_deploy_factory(provider_type):
     return UnitMachineDeployment
 
 
-def _get_environment(unit_name, juju_home, machine_id, zookeeper_hosts):
+def _get_environment(unit_name,
+                     juju_home, machine_id, zookeeper_hosts, env_id):
+    """
+    Return environment dictionary for unit.
+    """
     environ = dict()
     environ["JUJU_MACHINE_ID"] = str(machine_id)
     environ["JUJU_UNIT_NAME"] = unit_name
     environ["JUJU_HOME"] = juju_home
     environ["JUJU_ZOOKEEPER"] = zookeeper_hosts
+    environ["JUJU_ENV_UUID"] = env_id
     environ["PYTHONPATH"] = ":".join(
         filter(None, [
             os.path.dirname(get_module_directory(juju)),
@@ -66,13 +71,14 @@ class UnitMachineDeployment(object):
             "juju-%s" % self.unit_path_name, use_sudo=True)
 
     @inlineCallbacks
-    def start(self, machine_id, zookeeper_hosts, bundle):
+    def start(self, env_id, machine_id, zookeeper_hosts, bundle):
         """Start a service unit agent."""
         self.unpack_charm(bundle)
         self.service.set_description(
             "Juju unit agent for %s" % self.unit_name)
         self.service.set_environ(_get_environment(
-            self.unit_name, self.juju_home, machine_id, zookeeper_hosts))
+            self.unit_name, self.juju_home, machine_id,
+            zookeeper_hosts, env_id))
         self.service.set_command(" ".join((
             "/usr/bin/python", "-m", self.unit_agent_module,
             "--nodaemon",
@@ -189,7 +195,7 @@ class UnitContainerDeployment(object):
                 (zk, port) = zk.split(':')
             else:
                 port = 2181
-            zks.append((zk,port))
+            zks.append((zk, port))
 
         cloud_init.set_zookeeper_hosts(zks)
         # XXX Very hard to access the provider's notion of network
@@ -228,7 +234,7 @@ class UnitContainerDeployment(object):
         returnValue((container, directory))
 
     @inlineCallbacks
-    def start(self, machine_id, zookeeper_hosts, bundle):
+    def start(self, env_id, machine_id, zookeeper_hosts, bundle):
         """Start the unit.
 
         Creates and starts an lxc container for the unit.
