@@ -8,6 +8,7 @@ from txzookeeper import ZookeeperClient
 from juju.errors import ProviderError, EnvironmentNotFound
 from juju.lib.lxc import LXCContainer, get_containers
 from juju.lib.zk import Zookeeper
+from juju.lib.port import get_open_port
 from juju.providers.common.base import MachineProviderBase
 from juju.providers.common.connect import ZookeeperConnect
 from juju.providers.common.utils import get_user_authorized_keys
@@ -20,7 +21,6 @@ from juju.providers.local.pkg import check_packages
 from juju.state.auth import make_identity
 from juju.state.initialize import StateHierarchy
 from juju.state.placement import LOCAL_POLICY
-from juju.state.utils import get_open_port
 
 
 log = logging.getLogger("juju.local-dev")
