@@ -1,10 +1,7 @@
 # Copyright 2012 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
-__all__ = [
-    'MaaSMachine',
-    'MachineProvider',
-    ]
+__all__ = ['MAASMachine', 'MachineProvider']
 
-from juju.providers.maas.machine import MaaSMachine
+from juju.providers.maas.machine import MAASMachine
 from juju.providers.maas.provider import MachineProvider
