@@ -37,9 +37,10 @@ class _TagHandler(object):
     def convert(self, tag_expression):
         """Extract set of names in tag_expression checking they all exist"""
         tags = set()
+        stripped_expression = tag_expression
         for c in (",", "&", "|", "!"):
-            tag_expression = tag_expression.replace(c, " ")
-        for word in tag_expression.strip().split():
+            stripped_expression = stripped_expression.replace(c, " ")
+        for word in stripped_expression.strip().split():
             tag = word.lower()
             if tag not in self.tag_names:
                 raise ValueError("tag %r does not exist" % (tag,))
@@ -78,7 +79,7 @@ class MachineProvider(MachineProviderBase):
         tags_info = yield self.maas_client.list_tags()
         handler = _TagHandler(tags_info)
         cs.register("maas-tags", converter=handler.convert,
-            comparer=handler.compare)
+                    comparer=handler.compare)
         returnValue(cs)
 
     def get_serialization_data(self):
