@@ -7,6 +7,9 @@ from twisted.internet.defer import inlineCallbacks, succeed
 
 from juju.errors import ConstraintError, MachinesNotFound, ProviderError
 from juju.lib.mocker import ANY
+from juju.lib import serializer
+from juju.machine.constraints import Constraints
+
 from juju.providers.maas import MachineProvider
 from juju.providers.maas.maas import MAASClient
 from juju.providers.maas.tests.testing import (
@@ -183,13 +186,35 @@ class TestMAASProvider(TestCase):
             {'name': "clawed", 'definition': "HAS claws", 'comment': ""},
             ]))
         self.mocker.replay()
+
         provider = MachineProvider("maasiv", CONFIG)
         provider.maas_client = mock_client
         cs = yield provider.get_constraint_set()
         bear = cs.parse(["maas-tags=clawed, furry"])
-        bear.can_satisfy(bear)
-        err = self.assertRaises(ConstraintError,
+
+        # Incomplete constraints (no series) can't satisify
+        self.assertFalse(bear.can_satisfy(bear))
+
+        self.assertEqual(set(["clawed", "furry"]), bear["maas-tags"])
+
+        err = self.assertRaises(
+            ConstraintError,
             cs.parse, ["maas-tags=furry, bouncy"])
-        self.assertEqual("Bad 'maas-tags' constraint 'furry, bouncy': "
+
+        self.assertEqual(
+            "Bad 'maas-tags' constraint 'furry, bouncy': "
             "tag 'bouncy' does not exist",
             str(err))
+
+        bear = bear.with_series("precise")
+
+        # Ensure we can roundtrip through serialization.
+        raw = serializer.dump(bear.data)
+        grizzly = cs.load(serializer.load(raw))
+
+        self.assertTrue(grizzly.can_satisfy(bear))
+        rodent = cs.parse(
+            ["maas-tags=clawed"]).with_series("raring")
+        self.assertFalse(rodent.can_satisfy(grizzly))
+        self.assertTrue(grizzly.can_satisfy(
+            rodent.with_series("precise")))
