@@ -114,6 +114,9 @@ class FakeMAASHTTPConnection(object):
         # List some nodes.
         elif "nodes/?op=list_allocated&id=" in self.url:
             return self.list_some_nodes()
+        # List tags.
+        elif self.url.endswith("/tags/?op=list"):
+            return self.list_tags()
         # Not recognized.
         else:
             raise AssertionError("Unknown API method called")
@@ -161,6 +164,9 @@ class FakeMAASHTTPConnection(object):
     def release_node(self):
         return defer.succeed(json.dumps(NODE_JSON[0]))
 
+    def list_tags(self):
+        return defer.succeed("[]")
+
 
 class FakeMAASHTTPConnectionWithNoAvailableNodes(FakeMAASHTTPConnection):
     """Special version of L{FakeMAASHTTPConnection} that fakes that no nodes
