@@ -391,30 +391,39 @@ class _NovaClient(object):
         return self.delete(["servers", server_id], code=204)
 
     def run_server(self, image_id, flavor_id, name, security_group_names=None,
-        user_data=None):
+                   user_data=None, scheduler_hints=None):
         server = {
             'name': name,
             'flavorRef': flavor_id,
             'imageRef': image_id,
         }
+        post_dict = {"server": server}
         if user_data is not None:
             server["user_data"] = base64.b64encode(user_data)
         if security_group_names is not None:
             server["security_groups"] = [{'name': n}
                 for n in security_group_names]
-        return self.post(["servers"], {'server': server},
+        if scheduler_hints is not None:
+            post_dict["OS-SCH-HNT:scheduler_hints"] = scheduler_hints
+        return self.post(["servers"], post_dict,
             root="server", code=202)
 
     def get_server_security_groups(self, server_id):
         d = self.get(
             ["servers", server_id, "os-security-groups"],
             root="security_groups")
-        # 2012-07-12: Workaround lack of this api in HP cloud
-        d.addErrback(
-            lambda f: self.get_server(server_id).addCallback(
-                operator.itemgetter("security_groups")))
+
+        # 2012-07-12: kt Workaround lack of this api in HP cloud
+        def _get_group_fallback(f):
+            log.debug("Falling back to older/diablo sec groups api")
+            return self.get_server(server_id).addCallback(
+                operator.itemgetter("security_groups"))
+        d.addErrback(_get_group_fallback)
         return d
 
+    def get_security_group_details(self, group_id):
+        return self.get(["os-security-groups", group_id], "security_group")
+
     def list_security_groups(self):
         return self.get(["os-security-groups"], "security_groups")
 
