@@ -39,10 +39,10 @@ class ProviderPortMgmtTests(OpenStackTestMixin, TestCase):
 
     def test_open_port(self):
         """Opening a port adds the rule to the appropriate security group"""
-        self.expect_nova_get("servers/1000/os-security-groups",
+        self.expect_nova_get(
+            "servers/1000/os-security-groups",
             response={'security_groups': [
-                    {'name': "juju-testing-1", 'id': 1},
-                ]})
+                {'name': "juju-testing-1", 'id': 1, 'rules': []}]})
         self.expect_create_rule(1, "tcp", 80)
         self.mocker.replay()
 
@@ -52,9 +52,34 @@ class ProviderPortMgmtTests(OpenStackTestMixin, TestCase):
 
         def _check_log(_):
             self.assertIn("Opened 80/tcp on machine '1000'",
-                log.getvalue())
+                          log.getvalue())
         return deferred.addCallback(_check_log)
 
+    def test_diablo_hpcloud_ccompatbility(self):
+        """Verify compatibility workarounds for hpcloud/diablo."""
+        self.expect_nova_get(
+            "servers/1000/os-security-groups",
+            response={'security_groups': [
+                {'name': "juju-testing-1", 'id': 1}]})
+
+        self.expect_nova_get(
+            "os-security-groups/1",
+            response={'security_group': {
+                'name': "juju-testing-1",
+                'id': 1,
+                'rules': [{
+                    'id': 1,
+                    'parent_group_id': 1,
+                    'ip_protocol': 'tcp',
+                    'from_port': 80,
+                    'to_port': 80}]}})
+
+        self.mocker.replay()
+
+        machine = NovaProviderMachine('1000', "server1000.testing.invalid")
+        deferred = self.get_provider().get_opened_ports(machine, "1")
+        return deferred.addCallback(self.assertEqual, set([(80, "tcp")]))
+
     def test_open_port_missing_group(self):
         """Missing security group raises an error on deleting port"""
         self.expect_nova_get("servers/1000/os-security-groups",
