@@ -1,3 +1,4 @@
+import logging
 from collections import namedtuple
 
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed, fail
@@ -11,6 +12,9 @@ from juju.state.service import ServiceStateManager, parse_service_name
 from juju.state.utils import YAMLState
 
 
+log = logging.getLogger("juju.state.hook")
+
+
 class RelationChange(
     namedtuple(
         "RelationChange",
@@ -103,16 +107,23 @@ class HookContext(StateBase):
             self._topology = yield self._read_topology()
         service = yield self.get_local_service()
         internal_service_id = service.internal_id
+        service_unit_state = yield self.get_local_unit_state()
         for info in self._topology.get_relations_for_service(
                 internal_service_id):
             service_info = info["service"]
-            relations.append(
-                ServiceRelationState(
-                    self._client,
-                    internal_service_id,
-                    info["relation_id"],
-                    info["scope"],
-                    **service_info))
+            relation = ServiceRelationState(
+                self._client,
+                internal_service_id,
+                info["relation_id"],
+                info["scope"],
+                **service_info)
+            # Verify that it has unit relation state defined in ZK
+            try:
+                yield relation.get_unit_state(service_unit_state)
+                relations.append(relation)
+            except UnitRelationStateNotFound:
+                log.debug("Ignoring partially constructed relation: %s",
+                          relation.relation_ident)
         returnValue(relations)
 
     @inlineCallbacks
