@@ -1,4 +1,5 @@
 import logging
+import uuid
 
 from twisted.internet.defer import inlineCallbacks
 from txzookeeper.client import ZOO_OPEN_ACL_UNSAFE
@@ -70,6 +71,7 @@ class StateHierarchy(object):
         # Setup default global settings information.
         settings = GlobalSettingsStateManager(self.client)
         yield settings.set_provider_type(self.provider_type)
+        yield settings.set_environment_id(uuid.uuid4().get_hex())
 
         # This must come last, since clients will wait on it.
         yield self.client.create("/initialized", acls=acls)
