@@ -1,7 +1,6 @@
 import logging
 from os.path import basename, dirname
 
-import yaml
 import zookeeper
 
 from twisted.internet.defer import (
@@ -9,6 +8,7 @@ from twisted.internet.defer import (
 
 from txzookeeper.utils import retry_change
 
+from juju.lib import serializer
 from juju.state.base import StateBase
 from juju.state.errors import (
     DuplicateEndpoints, IncompatibleEndpoints, RelationAlreadyExists,
@@ -310,7 +310,7 @@ class ServiceRelationState(StateBase):
         except zookeeper.NodeExistsException:
             pass
 
-        node_data = yaml.safe_dump(relation_data)
+        node_data = serializer.dump(relation_data)
         role_path = "%s/%s" % (scope_path, relation_data["role"])
 
         yield retry_change(
@@ -346,18 +346,18 @@ class ServiceRelationState(StateBase):
         def update_address(content, stat):
             unit_map = None
             if content:
-                unit_map = yaml.load(content)
+                unit_map = serializer.load(content)
             if not unit_map:
                 unit_map = {}
             unit_map["private-address"] = private_address
-            return yaml.safe_dump(unit_map)
+            return serializer.dump(unit_map)
 
         yield retry_change(self._client, settings_path, update_address)
 
         # Update the unit name -> id mapping on the relation node
         def update_unit_mapping(content, stat):
             if content:
-                unit_map = yaml.load(content)
+                unit_map = serializer.load(content)
             else:
                 unit_map = {}
             # If it's already present, we're done, just return the
@@ -366,7 +366,7 @@ class ServiceRelationState(StateBase):
             if unit_state.internal_id in unit_map:
                 return content
             unit_map[unit_state.internal_id] = unit_state.unit_name
-            return yaml.dump(unit_map)
+            return serializer.dump(unit_map)
 
         yield retry_change(self._client,
                            "/relations/%s" % self._relation_id,
@@ -481,7 +481,7 @@ class UnitRelationState(StateBase):
         """
         path = yield self.get_settings_path()
         # encode as a YAML string
-        data = yaml.safe_dump(data)
+        data = serializer.dump(data)
 
         yield retry_change(
             self._client, path, lambda content, stat: data)
@@ -724,7 +724,7 @@ class RelationUnitWatcherBase(StateBase):
             if self._relation_scope == "container":
                 relation_path = dirname(relation_path)
             content, stat = yield self._client.get(relation_path)
-            self._unit_name_map = yaml.load(content)
+            self._unit_name_map = serializer.load(content)
         results = []
         for unit_id_list in unit_ids:
             names = []
