@@ -1,4 +1,3 @@
-import yaml
 import zookeeper
 
 from twisted.internet.defer import (
@@ -7,19 +6,24 @@ from twisted.internet.defer import (
 from txzookeeper.utils import retry_change
 
 from juju.charm.url import CharmURL
-from juju.machine.constraints import Constraints
+from juju.lib import serializer
 from juju.state.agent import AgentStateMixin
-from juju.state.base import StateBase
+from juju.state.base import log, StateBase
 from juju.state.endpoint import RelationEndpoint
+from juju.state.environment import EnvironmentStateManager
 from juju.state.errors import (
     StateChanged, ServiceStateNotFound, ServiceUnitStateNotFound,
     ServiceUnitStateMachineAlreadyAssigned, ServiceStateNameInUse,
     BadDescriptor, BadServiceStateName, NoUnusedMachines,
     ServiceUnitDebugAlreadyEnabled, ServiceUnitResolvedAlreadyEnabled,
-    ServiceUnitRelationResolvedAlreadyEnabled, StopWatcher)
+    ServiceUnitRelationResolvedAlreadyEnabled, StopWatcher,
+    IllegalSubordinateMachineAssignment, PrincipalServiceUnitRequired,
+    NotSubordinateCharm, ServiceUnitUpgradeAlreadyEnabled)
+
 from juju.state.charm import CharmStateManager
 from juju.state.relation import ServiceRelationState, RelationStateManager
 from juju.state.machine import _public_machine_id, MachineState
+from juju.state.topology import InternalTopologyError
 from juju.state.utils import (
     remove_tree, dict_merge, YAMLState, YAMLStateNodeMixin)
 
@@ -41,7 +45,7 @@ class ServiceStateManager(StateBase):
 
         @param service_name: Unique name of the service created.
         @param charm_state: CharmState for the service.
-        @param constraint_strs: list of constraint strings
+        @param constraints: Constraints needed to deploy the service.
 
         @return: ServiceState for the created service.
         """
@@ -51,13 +55,14 @@ class ServiceStateManager(StateBase):
             "charm": charm_id, "constraints": constraints.data}
         # charm metadata is always decoded into unicode, ensure any
         # serialized state references strings to avoid tying to py runtime.
-        node_data = yaml.safe_dump(service_details)
+        node_data = serializer.dump(service_details)
         path = yield self._client.create("/services/service-", node_data,
                                          flags=zookeeper.SEQUENCE)
         internal_id = path.rsplit("/", 1)[1]
 
         # create a child node for configuration options
-        yield self._client.create("%s/config" % path, yaml.dump({}))
+        yield self._client.create(
+            "%s/config" % path, serializer.dump({}))
 
         def add_service(topology):
             if topology.find_service_with_name(service_name):
@@ -187,16 +192,17 @@ class ServiceStateManager(StateBase):
                             service_name=query_service_name,
                             relation_type=spec["interface"],
                             relation_name=relation_name,
-                            relation_role=relation_role))
+                            relation_role=relation_role,
+                            relation_scope=spec["scope"]))
 
         # Add in implicit relations
         endpoints.add(
             RelationEndpoint(
                 query_service_name, "juju-info", "juju-info", "server"))
 
-        # when offering matches implicit relations should be considered last.
-        # this cmpfunc pushes implicit methods to the end of the list of possible
-        # endpoints
+        # When offering matches implicit relations should be
+        # considered last. This cmpfunc pushes implicit methods to
+        # the end of the list of possible endpoints.
         def low_priority_implicit_cmp(rel1, rel2):
             if rel1.relation_name.startswith("juju-"):
                 return 1
@@ -334,12 +340,21 @@ class ServiceState(StateBase, YAMLStateNodeMixin):
         charm = yield formula_state_manager.get_charm_state(charm_id)
         returnValue(charm)
 
+    @inlineCallbacks
+    def is_subordinate(self):
+        charm_state = yield self.get_charm_state()
+        returnValue(charm_state.is_subordinate())
+
     @inlineCallbacks
     def set_constraints(self, constraints):
         """Set hardware requirements for any new machines running this service.
 
-        :param constraint_strs: a list of constraint strings describing the
-            *service*-level requirements for new hardware.
+        :param constraints: a Constraints instance describing the service-level
+            machine constraints.
+
+        Constraints are settable individually by level, and dynamically
+        combined with the current environment constraints when it's time to
+        add a unit.
         """
         charm_id = yield self.get_charm_id()
         constraints = _series_constraints(constraints, charm_id)
@@ -347,35 +362,80 @@ class ServiceState(StateBase, YAMLStateNodeMixin):
 
     @inlineCallbacks
     def get_constraints(self):
-        """Get *service*-level constraints for new hardware.
+        """Get combined environment- and service-level machine constraints.
 
-        :return: a list of constraint strings describing the service-level
-            hardware constraints.
+        :return: a Constraints instance
+
+        This combined Constraints is used both to set the unit constraints at
+        add-unit time (so the unit gets a snapshot of the constraints in play
+        at creation time) and to display to the user in get-constraints (so
+        the user gets to see what constraints will actually be used to deploy
+        a unit, and what constraints are currently in play and hence need to
+        be set when changing service constraints).
         """
-        data = yield self._get_node_value("constraints")
-        returnValue(Constraints(data))
+        esm = EnvironmentStateManager(self._client)
+        constraints = yield esm.get_constraints()
+        constraint_set = yield esm.get_constraint_set()
+        service_data = yield self._get_node_value("constraints", {})
+        constraints.update(constraint_set.load(service_data))
+        returnValue(constraints)
 
     @inlineCallbacks
-    def add_unit_state(self):
+    def _validate_principal_container(self, principal_service_unit):
+        if not isinstance(principal_service_unit, ServiceUnitState):
+            raise PrincipalServiceUnitRequired(
+                self.service_name,
+                principal_service_unit)
+
+        principal_service = yield principal_service_unit.get_service_state()
+        if (yield principal_service.is_subordinate()):
+            raise PrincipalServiceUnitRequired(
+                self.service_name,
+                principal_service_unit.unit_name)
+
+        if not (yield self.is_subordinate()):
+            raise NotSubordinateCharm(self.service_name,
+                                      principal_service.unit_name)
+
+    @inlineCallbacks
+    def add_unit_state(self, container=None):
         """Add a new service unit to this state.
 
+        When provided container should be the service unit state of the
+        container for the unit state being added.
+
         @return: ServiceUnitState for the created unit.
+
+        The new unit's constraints are a frozen copy of the service's
+        actualised constraints at the time of unit addition (so, they don't
+        change in response to changes in the service or environment
+        constraints; if they did it would lead to extreme confusion (ie, after
+        a notable change to env/service, users would potentially see units
+        apparently deployed on wildly inappropriate machines)).
         """
         constraints = yield self.get_constraints()
         charm_id = yield self.get_charm_id()
         unit_data = {"charm": charm_id, "constraints": constraints.data}
         path = yield self._client.create(
-            "/units/unit-", yaml.dump(unit_data), flags=zookeeper.SEQUENCE)
+            "/units/unit-",
+            serializer.dump(unit_data),
+            flags=zookeeper.SEQUENCE)
 
         internal_unit_id = path.rsplit("/", 1)[1]
 
         sequence = [None]
+        if container is not None:
+            yield self._validate_principal_container(container)
+            container_id = container.internal_id
+        else:
+            container_id = None
 
         def add_unit(topology):
             if not topology.has_service(self._internal_id):
                 raise StateChanged()
             sequence[0] = topology.add_service_unit(self._internal_id,
-                                                    internal_unit_id)
+                                                    internal_unit_id,
+                                                    container_id)
 
         yield self._retry_topology_change(add_unit)
 
@@ -510,10 +570,10 @@ class ServiceState(StateBase, YAMLStateNodeMixin):
                 if old_relations:
                     old_relations = _to_service_relation_state(
                         self._client, self._internal_id, old_relations)
-                return callback(
-                    old_relations,
-                    _to_service_relation_state(
-                        self._client, self._internal_id, new_relations))
+                new_relations = _to_service_relation_state(
+                    self._client, self._internal_id, new_relations)
+
+                return callback(old_relations, new_relations)
 
         return self._watch_topology(watch_topology)
 
@@ -717,11 +777,13 @@ def _to_service_relation_state(client, service_id, assigned_relations):
            service relation specific information (role and name).
     """
     service_relations = []
-    for relation_id, relation_type, relation_info in assigned_relations:
+    for relation in assigned_relations:
+        relation_info = relation["service"]
         service_relations.append(
             ServiceRelationState(client,
                                  service_id,
-                                 relation_id,
+                                 relation["relation_id"],
+                                 relation["scope"],
                                  **relation_info))
     return service_relations
 
@@ -792,6 +854,44 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
     def _node_missing(self):
         raise ServiceUnitStateNotFound(self._service_name)
 
+        return "/units/%s/agent" % self._internal_id
+
+    @inlineCallbacks
+    def is_subordinate(self):
+        service_state = yield self.get_service_state()
+        returnValue((yield service_state.is_subordinate()))
+
+    @inlineCallbacks
+    def get_container(self):
+        """Return the ServiceUnitState of the container this unit executes in.
+
+        By default units are their own container, however subordinate services
+        will return the unit state of their principal service.
+
+        """
+        topology = yield self._read_topology()
+        try:
+            container_info = topology.get_service_unit_container(
+                self.internal_id)
+        except InternalTopologyError:
+            returnValue(None)
+
+        if container_info is None:
+            returnValue(None)
+
+        service_id, service_name, sequence, container_id = container_info
+        returnValue(ServiceUnitState(self._client,
+                                     service_id,
+                                     service_name,
+                                     sequence,
+                                     container_id))
+
+    def get_service_state(self):
+        """Return the service state for this unit."""
+        return ServiceState(self._client,
+                            self._internal_service_id,
+                            self._service_name)
+
     def get_public_address(self):
         """Get the public address of the unit.
 
@@ -835,9 +935,19 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
 
     @inlineCallbacks
     def get_constraints(self):
-        """The hardware constraints for this unit"""
-        data = yield self._get_node_value("constraints")
-        returnValue(Constraints(data))
+        """The complete constraints for this unit (as set at creation time).
+
+        Unit constraints are not settable; they're a snapshot of environment/
+        service state at unit addition time, so that get-constraints returns
+        sane results when asked about a unit when considered relative to the
+        constraints of the machine it's deployed on (rather than showing
+        combined service/env constraints for the parent, which could be wildly
+        different, and would certainly lead to confusion and bug reports).
+        """
+        esm = EnvironmentStateManager(self._client)
+        constraint_set = yield esm.get_constraint_set()
+        data = yield self._get_node_value("constraints", {})
+        returnValue(constraint_set.load(data))
 
     @inlineCallbacks
     def get_assigned_machine_id(self):
@@ -847,13 +957,26 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
         self._check_valid_in(topology)
         machine_id = topology.get_service_unit_machine(
             self._internal_service_id, self._internal_id)
+        if machine_id is None:
+            container = yield self.get_container()
+            if container:
+                machine_id = yield container.get_assigned_machine_id()
+                returnValue(machine_id)
+
         if machine_id is not None:
             machine_id = _public_machine_id(machine_id)
         returnValue(machine_id)
 
+    @inlineCallbacks
     def assign_to_machine(self, machine_state):
         """Assign this service unit to the given machine.
         """
+        constraints = yield self.get_constraints()
+        machine_constraints = yield machine_state.get_constraints()
+        if not machine_constraints.can_satisfy(constraints):
+            log.warning(
+                "Unit %s assigned to machine %s with incompatible constraints",
+                self.unit_name, machine_state.id)
 
         def assign_unit(topology):
             self._check_valid_in(topology)
@@ -861,6 +984,10 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
             machine_id = topology.get_service_unit_machine(
                 self._internal_service_id, self._internal_id)
 
+            container = topology.get_service_unit_container(self.internal_id)
+            if container is not None:
+                raise IllegalSubordinateMachineAssignment(self.unit_name)
+
             if machine_id is None:
                 topology.assign_service_unit_to_machine(
                     self._internal_service_id,
@@ -872,7 +999,7 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
                 pass
             else:
                 raise ServiceUnitStateMachineAlreadyAssigned(self.unit_name)
-        return self._retry_topology_change(assign_unit)
+        yield self._retry_topology_change(assign_unit)
 
     @inlineCallbacks
     def assign_to_unused_machine(self):
@@ -970,7 +1097,7 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
         try:
             yield self._client.create(
                 self._hook_debug_path,
-                yaml.safe_dump({"debug_hooks": hook_names}),
+                serializer.dump({"debug_hooks": hook_names}),
                 flags=zookeeper.EPHEMERAL)
         except zookeeper.NodeExistsException:
             raise ServiceUnitDebugAlreadyEnabled(self.unit_name)
@@ -1001,7 +1128,7 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
         except zookeeper.NoNodeException:
             # We get to the same end state.
             returnValue(None)
-        returnValue(yaml.load(content))
+        returnValue(serializer.load(content))
 
     @inlineCallbacks
     def watch_hook_debug(self, callback, permanent=True):
@@ -1047,21 +1174,41 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
         return "%s/upgrade" % self._zk_path
 
     @inlineCallbacks
-    def set_upgrade_flag(self):
+    def set_upgrade_flag(self, force=False):
         """Inform the unit it should perform an upgrade.
         """
-        try:
-            yield self._client.create(self._upgrade_flag_path)
-        except zookeeper.NodeExistsException:
-            # We get to the same end state
-            pass
+        assert isinstance(force, bool), "Invalid force upgrade flag"
+
+        def update(content, stat):
+            if not content:
+                flags = dict(force=force)
+                return serializer.dump(flags)
+
+            flags = serializer.load(content)
+            if not isinstance(flags, dict):
+                flags = dict(force=force)
+                return serializer.dump(flags)
+
+            if flags['force'] != force:
+                raise ServiceUnitUpgradeAlreadyEnabled(self.unit_name)
+
+            return content
+
+        yield retry_change(self._client,
+                           self._upgrade_flag_path,
+                           update)
 
     @inlineCallbacks
     def get_upgrade_flag(self):
-        """Returns a boolean denoting if the upgrade flag is set.
+        """Returns a dictionary containing the upgrade flag or False.
         """
-        stat = yield self._client.exists(self._upgrade_flag_path)
-        returnValue(bool(stat))
+        try:
+            content, stat = yield self._client.get(self._upgrade_flag_path)
+            if not content:
+                returnValue(False)
+        except zookeeper.NoNodeException:
+            returnValue(False)
+        returnValue(serializer.load(content))
 
     @inlineCallbacks
     def clear_upgrade_flag(self):
@@ -1138,7 +1285,7 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
 
         try:
             yield self._client.create(
-                self._unit_resolved_path, yaml.safe_dump({"retry": retry}))
+                self._unit_resolved_path, serializer.dump({"retry": retry}))
         except zookeeper.NodeExistsException:
             raise ServiceUnitResolvedAlreadyEnabled(self.unit_name)
 
@@ -1155,7 +1302,7 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
         except zookeeper.NoNodeException:
             # Return a default value.
             returnValue(None)
-        returnValue(yaml.load(content))
+        returnValue(serializer.load(content))
 
     @inlineCallbacks
     def clear_resolved(self):
@@ -1248,10 +1395,10 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
 
         def update_relation_resolved(content, stat):
             if not content:
-                return yaml.safe_dump(relation_map)
+                return serializer.dump(relation_map)
 
-            content = yaml.safe_dump(
-                dict_merge(yaml.load(content), relation_map))
+            content = serializer.dump(
+                dict_merge(serializer.load(content), relation_map))
             return content
 
         try:
@@ -1272,7 +1419,7 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
                 self._relation_resolved_path)
         except zookeeper.NoNodeException:
             returnValue(None)
-        returnValue(yaml.load(content))
+        returnValue(serializer.load(content))
 
     @inlineCallbacks
     def clear_relation_resolved(self):
@@ -1333,14 +1480,14 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
             if content is None:
                 data = {}
             else:
-                data = yaml.load(content)
+                data = serializer.load(content)
                 if data is None:
                     data = {}
             open_ports = data.setdefault("open", [])
             port_proto = dict(port=port, proto=proto)
             if port_proto not in open_ports:
                 open_ports.append(port_proto)
-            return yaml.safe_dump(data)
+            return serializer.dump(data)
 
         yield retry_change(self._client, self._ports_path, zk_open_port)
 
@@ -1355,14 +1502,14 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
             if content is None:
                 data = {}
             else:
-                data = yaml.load(content)
+                data = serializer.load(content)
                 if data is None:
                     data = {}
             open_ports = data.setdefault("open", [])
             port_proto = dict(port=port, proto=proto)
             if port_proto in open_ports:
                 open_ports.remove(port_proto)
-            return yaml.dump(data)
+            return serializer.dump(data)
 
         yield retry_change(
             self._client, self._ports_path, zk_close_port)
@@ -1380,7 +1527,7 @@ class ServiceUnitState(StateBase, AgentStateMixin, YAMLStateNodeMixin):
             content, stat = yield self._client.get(self._ports_path)
         except zookeeper.NoNodeException:
             returnValue([])
-        data = yaml.load(content)
+        data = serializer.load(content)
         if data is None:
             returnValue(())
         returnValue(data.get("open", ()))
