@@ -140,11 +140,24 @@ class GlobalSettingsTest(StateTestBase):
         """Debug logging is off by default."""
         self.assertEqual((yield self.manager.get_provider_type()), None)
         yield self.manager.set_provider_type("ec2")
+        yield self.assertFailure(
+            self.manager.set_provider_type("abc"), ValueError)
         self.assertEqual((yield self.manager.get_provider_type()), "ec2")
         content, stat = yield self.client.get("/settings")
         self.assertEqual(serializer.load(content),
                          {"provider-type": "ec2"})
 
+    @inlineCallbacks
+    def test_set_get_environment(self):
+        self.assertEqual((yield self.manager.get_environment_id()), None)
+        yield self.manager.set_environment_id('snowflake')
+        yield self.assertFailure(
+            self.manager.set_environment_id('snowflake'),
+            ValueError)
+        self.assertEqual(
+            (yield self.manager.get_environment_id()),
+            "snowflake")
+
     @inlineCallbacks
     def test_get_debug_log_enabled_no_settings_default(self):
         """Debug logging is off by default."""
