@@ -3,10 +3,9 @@ import logging
 from twisted.internet.defer import (
     Deferred, inlineCallbacks, fail, returnValue, succeed)
 
-from juju.environment.config import EnvironmentsConfig
 from juju.errors import ProviderInteractionError
 from juju.lib.mocker import MATCH
-from juju.machine.constraints import Constraints
+from juju.machine.tests.test_constraints import series_constraints
 from juju.providers.dummy import DummyMachine, MachineProvider
 from juju.state.errors import StopWatcher
 from juju.state.firewall import FirewallManager
@@ -17,14 +16,6 @@ from juju.state.tests.test_service import ServiceStateManagerTestBase
 
 MATCH_MACHINE = MATCH(lambda x: isinstance(x, DummyMachine))
 
-SAMPLE_ENV = """\
-environments:
-  myfirstenv:
-    type: dummy
-    foo: bar
-    storage-directory: %s
-"""
-
 
 class FirewallTestBase(ServiceStateManagerTestBase):
 
@@ -32,21 +23,13 @@ class FirewallTestBase(ServiceStateManagerTestBase):
     def setUp(self):
         yield super(FirewallTestBase, self).setUp()
         self._running = True
-        self.environment = yield self.get_test_environment()
+        self.environment = self.config.get_default()
         self.provider = self.environment.get_machine_provider()
-        self.firewall_manager = FirewallManager(self.client, self.is_running, self.provider)
+        self.firewall_manager = FirewallManager(
+            self.client, self.is_running, self.provider)
         self.service_state_manager = ServiceStateManager(self.client)
         self.output = self.capture_logging(level=logging.DEBUG)
 
-    def get_test_environment_config(self):
-        sample_config = SAMPLE_ENV % self.makeDir()
-        config = EnvironmentsConfig()
-        config.parse(sample_config)
-        return config
-
-    def get_test_environment(self):
-        return self.get_test_environment_config().get_default()
-
     # The following methods are used to provide the scaffolding given
     # by the provisioning agent, which normally runs the FirewallManager
 
@@ -279,8 +262,7 @@ class FirewallMachineTest(FirewallTestBase):
 
     def add_machine_state(self):
         manager = MachineStateManager(self.client)
-        constraints = Constraints.from_strs("dummy", []).with_series("series")
-        return manager.add_machine_state(constraints)
+        return manager.add_machine_state(series_constraints)
 
     @inlineCallbacks
     def get_provider_ports(self, machine):
@@ -384,6 +366,42 @@ class FirewallMachineTest(FirewallTestBase):
         self.assertEqual((yield self.get_provider_ports(machine)),
                          set([(80, "tcp"), (443, "tcp")]))
 
+    @inlineCallbacks
+    def test_immediately_close_previously_opened_ports(self):
+        """Verify machine with already open ports is closed upon provisioning.
+
+        This can happen because of the security group associated with
+        the machine is reused.
+        """
+        # Get a machine from the provider, ensure that it already has
+        # open ports by manually opening the port. This could be done
+        # by reuse, or an administrator hand tweaking the security
+        # group in EC2 itself, if using the EC2 provider
+        machine = yield self.add_machine_state()
+        provider_machine, = yield self.provider.start_machine(
+            {"machine-id": machine.id})
+        yield machine.set_instance_id(provider_machine.instance_id)
+        yield self.provider.open_port(provider_machine, machine.id, 53, "tcp")
+        self.assertEqual((yield self.get_provider_ports(machine)),
+                         set([(53, "tcp")]))
+
+        # Assign wordpress/0 to this machine and apply firewall policy
+        wordpress = yield self.add_service("wordpress")
+        yield wordpress.set_exposed_flag()
+        wordpress_0 = yield wordpress.add_unit_state()
+        yield wordpress_0.open_port(80, "tcp")
+        yield wordpress_0.open_port(443, "tcp")
+        yield wordpress_0.assign_to_machine(machine)
+        yield self.firewall_manager.process_machine(machine)
+
+        # Verify the initial manual open of 53/tcp, followed by the
+        # close due to the firewall processing
+        self.assertLogLines(self.output.getvalue(),
+                            ["Opened 53/tcp on provider machine 0",
+                             "Closed 53/tcp on provider machine 0"])
+        self.assertEqual((yield self.get_provider_ports(machine)),
+                         set([(80, "tcp"), (443, "tcp")]))
+
     @inlineCallbacks
     def test_open_close_ports_in_stopped_agent_stops_watch(self):
         """Verify code called by watches properly stops when agent stops."""
