@@ -1,3 +1,5 @@
+import logging
+
 from twisted.internet.defer import inlineCallbacks, returnValue
 from juju.lib.pick import pick_attr
 from juju.lib import serializer
@@ -98,23 +100,61 @@ class HookContextTestBase(RelationTestBase):
         self.relation = self.mysql_states["relation"]
 
     @inlineCallbacks
-    def add_another_blog(self, service_name):
-        blog_ep = RelationEndpoint(
-            service_name, "client-server", "database", "client")
-        mysql_ep = RelationEndpoint(
-            "mysql", "client-server", "db", "server")
-        yield self.add_relation_service_unit_from_endpoints(
-            blog_ep, mysql_ep)
+    def add_another_blog(self, blog_name):
+        blog_ep = RelationEndpoint(blog_name, "client-server", "app", "client")
+        # Fully construct states for the relation connecting to this additional blog
+        other_mysql_states = yield self.add_relation_service_unit_to_another_endpoint(
+                    self.mysql_states, blog_ep)
+        # Then complete in the opposite direction
+        blog_states = yield self.add_opposite_service_unit(other_mysql_states)
+        yield blog_states['service_relations'][-1].add_unit_state(
+            self.mysql_states['unit'])
+        returnValue(blog_states)
 
     @inlineCallbacks
     def add_db_admin_tool(self, admin_name):
         """Add another relation, using a different relation name"""
         admin_ep = RelationEndpoint(
             admin_name, "client-server", "admin-app", "client")
-        mysql_ep = RelationEndpoint(
+        mysql_admin_ep = RelationEndpoint(
             "mysql", "client-server", "db-admin", "server")
-        yield self.add_relation_service_unit_from_endpoints(
-            admin_ep, mysql_ep)
+        mysql_admin_states = yield self.reuse_service_unit_in_new_relation(
+            self.mysql_states, mysql_admin_ep, admin_ep)
+        admin_states = yield self.add_opposite_service_unit(mysql_admin_states)
+        returnValue(admin_states)
+
+    @inlineCallbacks
+    def reuse_service_unit_in_new_relation(self, reused_states, *endpoints):
+        """Reuse an existing service unit as part of a new relation"""
+        service_state = reused_states["service"]
+        unit_state = reused_states["unit"]
+
+        # 1. Setup all service states
+        service_states = [service_state]
+        for endpoint in endpoints[1:]:
+            service_state = yield self.add_service(endpoint.service_name)
+            service_states.append(service_state)
+
+        # 2. And join together in a relation
+        relation_state, service_relation_states = \
+                        yield self.relation_manager.add_relation_state(
+                            *endpoints)
+        # 3. Add a service unit to only the first endpoint - we need
+        #    to test what happens when service units are added to the
+        #    other service state (if any), so do so separately
+        relation_unit_state = yield service_relation_states[0].add_unit_state(
+            unit_state)
+
+        returnValue({
+            "endpoints": list(endpoints),
+            "service": service_states[0],
+            "services": service_states,
+            "unit": unit_state,
+            "relation": relation_state,
+            "service_relation": service_relation_states[0],
+            "unit_relation": relation_unit_state,
+            "service_relations": service_relation_states})
+
 
 
 class HookContextTest(HookContextTestBase, CommonHookContextTestsMixin):
@@ -162,6 +202,35 @@ class HookContextTest(HookContextTestBase, CommonHookContextTestsMixin):
             set((yield new_context.get_relation_idents(None))),
             set(["db:0", "db:1", "db:2", "db-admin:3"]))
 
+    @inlineCallbacks
+    def test_get_relation_idents_partial_updates_to_zk(self):
+        """Verify relation idents do not reflect partial updates to ZK."""
+        log = self.capture_logging(level=logging.DEBUG)
+
+        # 1. Partial update - no corresponding service relation unit
+        yield self.add_relation_service_unit_to_another_endpoint(
+            self.mysql_states,
+            RelationEndpoint(
+                "wordpress2", "client-server", "app", "client"))
+
+        # 2. Do a complete update of adding another relation and
+        # corresponding units
+        yield self.add_another_blog("wordpress3")
+        context = self.get_context("mysql/0")
+
+        # 3. Observe only the relation ids for wordpress, wordpress3
+        self.assertEqual(
+            set((yield context.get_relation_idents("db"))),
+            set(["db:0", "db:2"]))
+        self.assertIn("Ignoring partially constructed relation: db:1",
+                      log.getvalue())
+
+        # 4. Finally, relation ids for a nonexistent relation are
+        #    still not seen, or cause an error.
+        self.assertEqual(
+            set((yield context.get_relation_idents("not-a-relation"))),
+            set())
+
 
 class RelationHookContextTest(HookContextTestBase,
                               CommonHookContextTestsMixin):
@@ -178,10 +247,6 @@ class RelationHookContextTest(HookContextTestBase,
             self.wordpress_states, "modified", "mysql/0")
 
     def get_context(self, states, change_type, unit_name):
-        change = RelationChange(
-            states["service_relation"].relation_ident,
-            change_type,
-            unit_name)
         return RelationHookContext(
             self.client, states["unit_relation"],
             states["service_relation"].relation_ident,
@@ -605,7 +670,7 @@ class RelationHookContextTest(HookContextTestBase,
         self.assertEqual(
             set((yield new_context.get_relation_idents("db"))),
             set(["db:1"]))
-        yield self.assertFailure( 
+        yield self.assertFailure(
             new_context.get_relation_hook_context("db:0"),
             RelationStateNotFound)
         db1 = yield new_context.get_relation_hook_context("db:1")
