@@ -1,28 +1,27 @@
 import functools
 
-import yaml
-
 from twisted.internet.defer import inlineCallbacks, Deferred, returnValue
 
 from juju.charm.tests import local_charm_id
 from juju.errors import ConstraintError
-from juju.machine.constraints import Constraints
+from juju.lib import serializer
+from juju.machine.tests.test_constraints import (
+    dummy_constraints, series_constraints)
 from juju.state.charm import CharmStateManager
 from juju.state.machine import MachineStateManager
 from juju.state.service import ServiceStateManager
 from juju.state.errors import MachineStateNotFound, MachineStateInUse
+from juju.state.utils import YAMLState
 
 from juju.state.tests.common import StateTestBase
 
-dummy_constraints = Constraints.from_strs("dummy", [])
-series_constraints = dummy_constraints.with_series("series")
-
 
 class MachineStateManagerTest(StateTestBase):
 
     @inlineCallbacks
     def setUp(self):
         yield super(MachineStateManagerTest, self).setUp()
+        yield self.push_default_config()
         self.charm_state_manager = CharmStateManager(self.client)
         self.machine_state_manager = MachineStateManager(self.client)
         self.service_state_manager = ServiceStateManager(self.client)
@@ -72,6 +71,19 @@ class MachineStateManagerTest(StateTestBase):
         self.assertEquals(
             str(e), "Unprovisionable machine: incomplete constraints")
 
+    @inlineCallbacks
+    def test_missing_constraints(self):
+        """ensure compatibility with nodes written for previous versions"""
+        yield self.add_machine_state()
+        machine = yield self.machine_state_manager.get_machine_state(0)
+        path = "/machines/" + machine.internal_id
+        node = YAMLState(self.client, path)
+        yield node.read()
+        del node["constraints"]
+        yield node.write()
+        constraints = yield machine.get_constraints()
+        self.assertEquals(constraints.data, {})
+
     @inlineCallbacks
     def test_machine_str_representation(self):
         """The str(machine) value includes the machine id.
@@ -224,7 +236,7 @@ class MachineStateManagerTest(StateTestBase):
 
         content, stat = yield self.client.get("/machines/machine-0000000000")
         self.assertEquals(
-            yaml.load(content)["provider-machine-id"], "custom-id")
+            serializer.load(content)["provider-machine-id"], "custom-id")
 
     @inlineCallbacks
     def test_set_instance_id_preserves_existing_data(self):
@@ -233,11 +245,11 @@ class MachineStateManagerTest(StateTestBase):
         """
         machine_state = yield self.add_machine_state()
         yield self.client.set("/machines/machine-0000000000",
-                              yaml.dump({"foo": "bar"}))
+                              serializer.dump({"foo": "bar"}))
         yield machine_state.set_instance_id("custom-id")
 
         content, stat = yield self.client.get("/machines/machine-0000000000")
-        self.assertEquals(yaml.load(content),
+        self.assertEquals(serializer.load(content),
                           {"provider-machine-id": "custom-id",
                            "foo": "bar"})
 
@@ -286,7 +298,7 @@ class MachineStateManagerTest(StateTestBase):
         """
         machine_state = yield self.add_machine_state()
         yield self.client.set("/machines/machine-0000000000",
-                              yaml.dump({"foo": "bar"}))
+                              serializer.dump({"foo": "bar"}))
 
         instance_id = yield machine_state.get_instance_id()
         self.assertEquals(instance_id, None)
