@@ -865,6 +865,9 @@ class ServiceRelationStateTest(RelationTestBase):
                                  "contents": {"private-address": None}},
                              "unit-0000000002": {
                                  "contents": {"private-address": None}}}}})
+        topology = yield self.get_topology()
+        self.assertTrue(
+            topology.has_relation_between_endpoints([mysql_ep, logging_ep]))
 
     @inlineCallbacks
     def test_presence_node_is_ephemeral(self):
