@@ -14,7 +14,7 @@ from juju.lib.testing import TestCase
 from juju.state.base import StateBase
 from juju.state.errors import StateChanged, StateNotFound
 from juju.state.utils import (
-    PortWatcher, remove_tree, dict_merge, get_open_port,
+    PortWatcher, remove_tree, dict_merge,
     YAMLState, YAMLStateNodeMixin, AddedItem, ModifiedItem, DeletedItem)
 
 
@@ -227,35 +227,6 @@ class DictMergeTest(TestCase):
             dict(b=2, c=1))
 
 
-class OpenPortTest(TestCase):
-
-    def test_get_open_port(self):
-        port = get_open_port()
-        self.assertTrue(isinstance(port, int))
-
-        sock = socket.socket(
-            socket.AF_INET, socket.SOCK_STREAM, socket.IPPROTO_TCP)
-
-        # would raise an error if we got it wrong.
-        sock.bind(("127.0.0.1", port))
-        sock.listen(1)
-        sock.close()
-        del sock
-
-    def test_get_open_port_with_host(self):
-        port = get_open_port("localhost")
-        self.assertTrue(isinstance(port, int))
-
-        sock = socket.socket(
-            socket.AF_INET, socket.SOCK_STREAM, socket.IPPROTO_TCP)
-
-        # would raise an error if we got it wrong.
-        sock.bind(("127.0.0.1", port))
-        sock.listen(1)
-        sock.close()
-        del sock
-
-
 class ChangeItemsTest(TestCase):
     """Tests the formatting of change items.
 
