@@ -544,11 +544,11 @@ class InternalTopology(object):
         for relation_data in relations.itervalues():
             scope = relation_data["scope"]
             services = relation_data["services"]
+
             for endpoint in endpoints:
                 service = services.get(service_ids[endpoint])
-                if not service or service["name"] != \
-                   endpoint.relation_name or \
-                   scope != endpoint.relation_scope:
+                if (not service or service["name"] != endpoint.relation_name or
+                    scope not in (endpoint.relation_scope, "container")):
                     break
             else:
                 return True
