@@ -360,6 +360,30 @@ class UnitLifecycle(object):
         finally:
             self._run_lock.release()
 
+    def _sort_relations(self, rel_ids, rels, invert=False):
+        """ Sort a set of relations.
+
+        We process peer relations first when adding, and last when
+        removing, else deferring to creation order.
+        """
+        rel_ids = list(rel_ids)
+
+        def _sort(x, y):
+            xr, yr = rels[x].relation_role, rels[y].relation_role
+            if xr == yr:
+                return cmp(x, y)
+            elif xr == "peer":
+                return -1
+            elif yr == "peer":
+                return 1
+            return cmp(x, y)
+
+        rel_ids.sort(_sort)
+
+        if invert:
+            return list(reversed(rel_ids))
+        return rel_ids
+
     @inlineCallbacks
     def _process_service_changes(self, old_relations, new_relations):
         """Add and remove unit lifecycles per the service relations Determine.
@@ -374,8 +398,15 @@ class UnitLifecycle(object):
                 (service_relation.internal_relation_id, service_relation)
                 for service_relation in old_relations)
 
-        added = set(new_relations.keys()) - set(self._relations.keys())
-        removed = set(self._relations.keys()) - set(new_relations.keys())
+        added = self._sort_relations(
+            set(new_relations.keys()) - set(self._relations.keys()),
+            new_relations)
+
+        removed = self._sort_relations(
+            set(self._relations.keys()) - set(new_relations.keys()),
+            self._relations,
+            invert=True)
+
         # Could this service be a principal container?
         is_principal = not (yield self._service.is_subordinate())
 
@@ -404,12 +435,13 @@ class UnitLifecycle(object):
                 yield workflow.transition_state("departed")
             self._store_relations()
 
-        # Process new relations.
+        # Process new relations
         for relation_id in added:
             service_relation = new_relations[relation_id]
             yield self._add_relation(service_relation)
-            if (is_principal and service_relation.relation_scope == "container"):
-                self._add_subordinate_unit(service_relation)
+            if (is_principal
+                    and service_relation.relation_scope == "container"):
+                yield self._add_subordinate_unit(service_relation)
             yield self._store_relations()
 
     @inlineCallbacks
