@@ -1,23 +1,18 @@
 import subprocess
-import zookeeper
 
 from twisted.internet.defer import inlineCallbacks, succeed, returnValue
 from twisted.web import client
 
 from juju.errors import JujuError
 from juju.lib.testing import TestCase
+from juju.state.tests.common import StateTestBase
 from juju.unit.address import (
     EC2UnitAddress, LocalUnitAddress, OrchestraUnitAddress, DummyUnitAddress,
     MAASUnitAddress, OpenStackUnitAddress, UnitAddress, get_unit_address)
 from juju.state.environment import GlobalSettingsStateManager
 
 
-class AddressTest(TestCase):
-
-    def setUp(self):
-        zookeeper.set_debug_level(0)
-        self.client = self.get_zookeeper_client()
-        return self.client.connect()
+class AddressTest(StateTestBase):
 
     @inlineCallbacks
     def get_address_for(self, provider_type):
