@@ -34,7 +34,7 @@ from juju.unit.lifecycle import (
 from juju.unit.tests.test_charm import CharmPublisherTestBase
 
 from juju.lib.testing import TestCase
-from juju.lib.mocker import MATCH
+from juju.lib.mocker import MATCH, ANY
 
 
 class UnwriteablePath(object):
@@ -64,10 +64,15 @@ class LifecycleTestBase(RelationTestBase):
                                              level=logging.DEBUG)
         self.agent_log = self.capture_logging("unit-agent",
                                               level=logging.DEBUG)
+        self.patch(
+            HookExecutor,
+            "LOCK_PATH",
+            os.path.join(self.makeDir(), "hook.lock"))
         self.executor = HookExecutor()
         self.executor.start()
         self.change_environment(
             PATH=os.environ["PATH"],
+            JUJU_ENV_UUID="snowflake",
             JUJU_UNIT_NAME="service-unit/0")
 
     @inlineCallbacks
@@ -888,6 +893,20 @@ class UnitLifecycleTest(LifecycleTestBase):
         self.assertEqual("up", (yield staying_workflow.get_state()))
         yield new_lifecycle.stop()
 
+    def test_multiple_rel_order(self):
+        """Given multiple new rels, verify peer processed first."""
+
+        class rel(object):
+            def __init__(self, r):
+                self.relation_role = r
+
+        rels = {'5': rel('peer'), '4': rel('client'),
+                '2': rel('server'), '1': rel('peer')}
+
+        self.assertEqual(
+            self.lifecycle._sort_relations(rels.keys(), rels),
+            ["1", "5", "2", "4"])
+
     @inlineCallbacks
     def test_start_invoker(self):
         """Verify the invoker is started by the unit relation lifecycle"""
@@ -896,15 +915,20 @@ class UnitLifecycleTest(LifecycleTestBase):
         # these will be on the relation with ident app:0, app:1, app:2
         self.write_hook(
             "start",
-            '#!/bin/sh\n echo "hello world"\n')
-        yield self.add_opposite_service_unit(self.states)
+            '#!/bin/sh\n echo hello\n')
+        blog_states = yield self.add_opposite_service_unit(self.states)
+        yield blog_states['service_relations'][-1].add_unit_state(
+            self.states['unit'])
+
         for i in range(1, 3):
-            yield self.add_opposite_service_unit(
+            blog_states = yield self.add_opposite_service_unit(
                 (yield self.add_relation_service_unit_to_another_endpoint(
-                        self.states,
-                        RelationEndpoint(
-                            "wordpress-%d" % i,
-                            "client-server", "db", "client"))))
+                    self.states,
+                    RelationEndpoint(
+                        "wordpress-%d" % i,
+                        "client-server", "db", "client"))))
+            yield blog_states['service_relations'][-1].add_unit_state(
+                self.states['unit'])
 
         finished = self.wait_on_hook("app-relation-changed", 3)
         yield self.lifecycle.start()
@@ -1006,6 +1030,7 @@ class RelationInvokerTest(TestCase):
         charm specification of hook invocation."""
         self.change_environment(
             PATH=os.environ["PATH"],
+            JUJU_ENV_UUID="snowflake",
             JUJU_UNIT_NAME="service-unit/0")
         change = RelationChange("clients:42", "joined", "s/2")
         unit_hook_path = self.makeDir()
@@ -1014,6 +1039,7 @@ class RelationInvokerTest(TestCase):
         self.assertEqual(environ["JUJU_RELATION"], "clients")
         self.assertEqual(environ["JUJU_RELATION_ID"], "clients:42")
         self.assertEqual(environ["JUJU_REMOTE_UNIT"], "s/2")
+        self.assertEqual(environ["JUJU_ENV_UUID"], "snowflake")
         self.assertEqual(environ["CHARM_DIR"],
                          os.path.join(unit_hook_path, "charm"))
 
@@ -1119,10 +1145,12 @@ class UnitRelationLifecycleTest(LifecycleTestBase):
 
         yield self.lifecycle.start()
         yield self.wait_on_hook("app-relation-joined")
-        self.assertEqual(len(errors), 1)
+
         # Give a chance for the relation-changed hook to erroneously fire.
         yield self.sleep(0.3)
 
+        self.assertEqual(len(errors), 1)
+
     @inlineCallbacks
     def test_initial_start_lifecycle_with_related(self):
         """
@@ -1328,14 +1356,20 @@ class UnitRelationLifecycleTest(LifecycleTestBase):
         # Setup 5 different wordpress services, wordpress, wordpress-1
         # through wordpress-4 with one service unit each. Each of these
         # will be on the relation app:0 ... app:4
-        yield self.add_opposite_service_unit(self.states)
+        blog_states = yield self.add_opposite_service_unit(self.states)
+        yield blog_states['service_relations'][-1].add_unit_state(
+            self.states['unit'])
+
         for i in range(1, 5):
-            yield self.add_opposite_service_unit(
+            blog_states = yield self.add_opposite_service_unit(
                 (yield self.add_relation_service_unit_to_another_endpoint(
                         self.states,
                         RelationEndpoint(
                             "wordpress-%d" % i,
                             "client-server", "db", "client"))))
+            yield blog_states['service_relations'][-1].add_unit_state(
+                self.states['unit'])
+
         yield self.lifecycle.start()
         yield self.wait_on_hook(
             sequence=["app-relation-joined", "app-relation-changed"])
