@@ -2,7 +2,6 @@ import csv
 import itertools
 import logging
 import os
-import yaml
 
 from twisted.internet.defer import inlineCallbacks, returnValue
 
@@ -12,6 +11,7 @@ from juju.unit.tests.test_lifecycle import LifecycleTestBase
 
 from juju.charm.directory import CharmDirectory
 from juju.charm.url import CharmURL
+from juju.lib import serializer
 from juju.lib.statemachine import WorkflowState
 from juju.unit.lifecycle import UnitLifecycle, UnitRelationLifecycle
 from juju.unit.workflow import (
@@ -40,10 +40,10 @@ class WorkflowTestBase(LifecycleTestBase):
 
         state = open(workflow.state_file_path).read()
         history = open(workflow.state_history_path)
-        zk_state = yaml.load(data)["workflow_state"]
-        returnValue((yaml.load(state),
-                    [yaml.load(r[0]) for r in csv.reader(history)],
-                     yaml.load(zk_state[history_id])))
+        zk_state = serializer.load(data)["workflow_state"]
+        returnValue((serializer.load(state),
+                    [serializer.load(r[0]) for r in csv.reader(history)],
+                     serializer.load(zk_state[history_id])))
 
     @inlineCallbacks
     def assert_history(self, expected, **kwargs):
@@ -529,7 +529,6 @@ class UnitWorkflowUpgradeTest(
             ("start", "started"),
             ("upgrade_charm", "started"))
 
-
     @inlineCallbacks
     def test_upgrade_error_retry(self):
         """A hook error during an upgrade transitions to
@@ -603,6 +602,7 @@ class UnitWorkflowUpgradeTest(
     def test_upgrade_error_before_hook(self):
         """If we blow up during the critical pre-hook bits, we should still
         end up in the same error state"""
+        self.capture_logging("charm.upgrade")
         yield self.assert_transition("install")
         yield self.assert_state("started")
         yield self.ready_upgrade(False)
@@ -625,6 +625,7 @@ class UnitWorkflowUpgradeTest(
         # charm only just got overwritten: and so we know that we've never even
         # tried to execute a hook for this upgrade, and we must do so to fulfil
         # the guarantee that that hook runs first after upgrade.
+
         self.assert_hooks(
             "install", "config-changed", "start", "upgrade-charm")
         yield self.assert_history_concise(
@@ -641,6 +642,7 @@ class UnitRelationWorkflowTest(WorkflowTestBase):
         yield super(UnitRelationWorkflowTest, self).setUp()
         yield self.setup_default_test_relation()
         self.relation_name = self.states["service_relation"].relation_name
+        self.relation_ident = self.states["service_relation"].relation_ident
         self.log_stream = self.capture_logging(
             "unit.relation.lifecycle", logging.DEBUG)
 
@@ -648,7 +650,7 @@ class UnitRelationWorkflowTest(WorkflowTestBase):
             self.client,
             self.states["unit"].unit_name,
             self.states["unit_relation"],
-            self.relation_name,
+            self.relation_ident,
             self.unit_directory,
             self.state_directory,
             self.executor)
@@ -673,16 +675,34 @@ class UnitRelationWorkflowTest(WorkflowTestBase):
                 self.client, self.states["unit_relation"])
             self.assertIdentical(running, False)
             self.assertIdentical(state, None)
+            relation_state = self.workflow.get_relation_info()
+            self.assertEquals(relation_state,
+                              {"relation-0000000000":
+                               {"relation_name": "mysql/0",
+                                "relation_scope": "global"}})
+
             yield self.workflow.fire_transition("start")
             running, state = yield is_relation_running(
                 self.client, self.states["unit_relation"])
             self.assertIdentical(running, True)
             self.assertEqual(state, "up")
+
+            relation_state = self.workflow.get_relation_info()
+            self.assertEquals(relation_state,
+                              {"relation-0000000000":
+                               {"relation_name": "mysql/0",
+                                "relation_scope": "global"}})
+
             yield self.workflow.fire_transition("stop")
             running, state = yield is_relation_running(
                 self.client, self.states["unit_relation"])
             self.assertIdentical(running, False)
             self.assertEqual(state, "down")
+            relation_state = self.workflow.get_relation_info()
+            self.assertEquals(relation_state,
+                              {"relation-0000000000":
+                               {"relation_name": "mysql/0",
+                                "relation_scope": "global"}})
 
     @inlineCallbacks
     def test_up_down_cycle(self):
@@ -778,7 +798,7 @@ class UnitRelationWorkflowTest(WorkflowTestBase):
         self.assertEqual(f_state,
                          {"state": "error",
                           "state_variables": {
-                              "change_type": "joined",
+                              "change_type": "modified",
                               "error_message": error}})
 
     @inlineCallbacks
@@ -906,7 +926,7 @@ class UnitRelationWorkflowTest(WorkflowTestBase):
     @inlineCallbacks
     def test_synchronize(self):
         # No transition in flight
-        yield self.assert_synchronize(None,  "up", True, True, False, False)
+        yield self.assert_synchronize(None, "up", True, True, False, False)
         yield self.assert_synchronize("down", "down", False, False)
         yield self.assert_synchronize("departed", "departed", False, False)
         yield self.assert_synchronize("error", "error", True, False)
