@@ -1,4 +1,3 @@
-import yaml
 import csv
 import os
 import logging
@@ -9,6 +8,7 @@ from twisted.internet.defer import inlineCallbacks, returnValue
 from txzookeeper.utils import retry_change
 
 from juju.errors import CharmError, FileNotFound
+from juju.lib import serializer
 from juju.lib.statemachine import (
     WorkflowState, Workflow, Transition, TransitionError)
 
@@ -223,16 +223,16 @@ class ZookeeperWorkflowState(WorkflowState):
     @inlineCallbacks
     def _store(self, state_dict):
         """Store the workflow state dictionary in zookeeper."""
-        state_serialized = yaml.safe_dump(state_dict)
+        state_serialized = serializer.dump(state_dict)
 
         def update_state(content, stat):
-            unit_data = yaml.load(content)
+            unit_data = serializer.load(content)
             if not unit_data:
                 unit_data = {}
 
             persistent_workflow = unit_data.setdefault("workflow_state", {})
             persistent_workflow[self.zk_state_id] = state_serialized
-            return yaml.dump(unit_data)
+            return serializer.dump(unit_data)
 
         yield retry_change(self._client, self.zk_state_path, update_state)
         yield super(ZookeeperWorkflowState, self)._store(
@@ -245,8 +245,8 @@ class ZookeeperWorkflowState(WorkflowState):
             data, stat = yield self._client.get(self.zk_state_path)
         except NoNodeException:
             returnValue({"state": None})
-        unit_data = yaml.load(data)
-        data = yaml.load(unit_data.get("workflow_state", {}).get(
+        unit_data = serializer.load(data)
+        data = serializer.load(unit_data.get("workflow_state", {}).get(
                 self.zk_state_id, ""))
         returnValue(data)
 
@@ -273,7 +273,7 @@ class DiskWorkflowState(ZookeeperWorkflowState):
         Internally the history file is stored a csv, with a new
         row per entry with CSV escaping.
         """
-        state_serialized = yaml.safe_dump(state_dict)
+        state_serialized = serializer.dump(state_dict)
         # State File
         with open(self.state_file_path, "w") as handle:
             handle.write(state_serialized)
@@ -295,7 +295,7 @@ class DiskWorkflowState(ZookeeperWorkflowState):
             content = handle.read()
 
         # TODO load ZK state and overwrite with disk state if different?
-        return yaml.load(content)
+        return serializer.load(content)
 
 
 class WorkflowStateClient(ZookeeperWorkflowState):
